# -*- coding: utf-8 -*-

from os import path

from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.DataAccess import DataAccess, QueryItem
from ImportarSurvey.Estadisticas import Estadisticas

class ImportarDatos:

    def __init__(self, 
        fuente_da, 
        destino_da, 
        claseFuentePrincipal = '', 
        claseDestinoPrincipal = '', 
        usuarioCampo = '') :

        self.fuente_da = fuente_da
        self.destino_da = destino_da
        self.claseFuentePrincipal = claseFuentePrincipal
        self.claseDestinoPrincipal = claseDestinoPrincipal
        self.usuarioCampo = usuarioCampo

        self.estadisticas = Estadisticas()
        self.parentGlobalId = ''

    def obtenerTablaPrincipal(self):
        return self.claseFuentePrincipal

    def nombreTabla(self, nombreLargoTabla):
        ruta, nombre = path.split(nombreLargoTabla)
        ruta = ruta

        return nombre

    def actualizarEstadistica(self, nombreTabla, tipo = "", guid = '{}'):
        self.estadisticas.actualizarEstadistica(nombreTabla, tipo, guid)

    def cumpleCondicion(self, condicion, registro):
        if condicion != None:
            condicion_campo = condicion["campo"] if "campo" in condicion else None
            condicion_valor = condicion["valor"] if "valor" in condicion else None

            if condicion_campo and condicion_valor:

                if isinstance(condicion_valor, list):
                    ToolboxLogger.debug("{} in {} <=> {} = {}".format(
                        condicion_campo, condicion_valor, condicion_campo, registro[condicion_campo]))
                    valor = [x for x in condicion_valor if x ==
                             registro[condicion_campo]]
                    if valor:
                        return True
                else:
                    ToolboxLogger.debug("Condicion -> {} = {} <=> {} = {}".format(
                        condicion_campo, condicion_valor, condicion_campo, registro[condicion_campo]))
                    return registro[condicion_campo] == condicion_valor
            return False
        else:
            return True

    @ToolboxLogger.log_method
    def obtenerValoresMapeados(self, campos, registro_origen) :
        geometria = registro_origen["geometry"] if "geometry" in registro_origen else None
        atributos = registro_origen["attributes"] if "attributes" in registro_origen else None

        registro = {}
        if campos:
            for campo in campos:
                if "origen" in campo :
                    valor = atributos[campo["origen"]]
                elif "valor" in campo :
                    valor = campo["valor"]
                else :
                    valor = None

                if "esGuid" in campo and valor != None:
                    valor = "{}".format(valor)
                
                if valor != None :
                    registro[campo["destino"]] = valor
                    ToolboxLogger.debug(
                        "---->mapeo - {} = {}".format(campo["destino"], valor)
                    )
        return registro, geometria
    
    def _obtenerCamposMapeo(self, mapeo, llaves) :
        campos = []

        def agregar(item) :
            if not item in campos and item != "ND":
                campos.append(item)

        if "relacion" in llaves and llaves["relacion"] in mapeo:
            agregar(mapeo[llaves["relacion"]])
        if "primaria" in llaves and llaves["primaria"] in mapeo :
            agregar(mapeo[llaves["primaria"]])
        if "foranea" in llaves and llaves["foranea"]  in mapeo :
            agregar(mapeo[llaves["foranea"]])
        if "hija" in llaves and "detallesRelacion" in mapeo and llaves["hija"] in mapeo["detallesRelacion"] :
            agregar(mapeo["detallesRelacion"][llaves["hija"]])
        if "padre" in llaves and "detallesRelacion" in mapeo and llaves["padre"] in mapeo["detallesRelacion"] :
            agregar(mapeo["detallesRelacion"][llaves["padre"]])
        if "campos" in mapeo :
            for campo_mapeo in mapeo["campos"] :
                if llaves["tipo"] in campo_mapeo :
                    agregar(campo_mapeo[llaves["tipo"]])
        return campos

    def _obtenerCamposOrigen(self, mapeo) :
        llaves = {"tipo": "origen", 
                  "relacion" : "llaveRelacionOrigen", 
                  "primaria" : "llavePrimariaOrigen",
                  "foranea" : "llaveForaneaOrigen"}
        return self._obtenerCamposMapeo(mapeo, llaves)

    def _obtenerCamposDestino(self, mapeo) :
        llaves = {"tipo": "destino", 
                  "relacion" : "llaveRelacionDestino", 
                  "primaria" : "llavePrimariaDestino",
                  "foranea" : "llaveForaneaDestino", 
                  "hija" : "llaveForaneaHija", 
                  "padre" : "llaveForaneaPadre"}
        return self._obtenerCamposMapeo(mapeo, llaves)
   
    @ToolboxLogger.log_method
    def crearRegistro(self, table, campos_busqueda, campos, registro_origen, valores_pk, usar_geometria=False):
        ToolboxLogger.debug("---->campos_busqueda: {}".format(campos_busqueda))
        ToolboxLogger.debug("---->valores_pk: {}".format(valores_pk))
        try:
            attributes = {}
            if isinstance(campos_busqueda, list) and isinstance(valores_pk, list):
                enum_campos = DataAccess.enumElements(campos_busqueda)
                enum_valores = DataAccess.enumElements(valores_pk)
                for enum_campo in enum_campos:
                    index = enum_campos.index(enum_campo)
                    attributes[enum_campo] = enum_valores[index]
                ToolboxLogger.debug("---->campos_busqueda: {}".format(enum_campos))
                ToolboxLogger.debug("---->valores_pk: {}".format(enum_valores))
            else:
                attributes[campos_busqueda] = valores_pk
            ToolboxLogger.debug("---->attributes: {}".format(attributes))

            if registro_origen and "geometry" in registro_origen:  
                attributes["geometry"] = registro_origen["geometry"]

            if campos:
                for campo in campos:
                    if "origen" in campo and campo["origen"] in registro_origen :
                        if campo["origen"] != "ND":
                            valor = registro_origen[campo["origen"]]
                        elif "destino" in campo:
                            if campo["origen"] == "ND" and campo["destino"].upper()=="PARENTGLOBALID":
                                valor = self.parentGlobalId
                    elif "valor" in campo :
                        valor = campo["valor"]
                    else :
                        valor = None

                    if valor == None and "valor_predeterminado" in campo:
                        valor = campo["valor_predeterminado"]    

                    if "esGuid" in campo and valor != None:
                        valor = "{}".format(valor)
                    
                    if valor != None :
                        if str(valor).strip() != '':
                            fieldToFill = [x for x in table.properties.fields if x.name.upper() == campo["destino"].upper()]
                            if len(fieldToFill)  > 0 :
                                if fieldToFill[0].type == 'esriFieldTypeDate' :
                                    attributes[campo["destino"]] = int(valor)  
                                else:
                                    attributes[campo["destino"]] = str(valor).strip()
                        else :
                            attributes[campo["destino"]] = None

                        ToolboxLogger.debug("---->ins - {} = {}".format(campo["destino"], valor))
                    else :
                        if table :
                            fields = [x for x in table.properties.fields if x.name.upper() == campo["destino"].upper()]
                            if len(fields)  > 0 :
                                field = fields[0]
                                if field.nullable == False and field.defaultValue != None :
                                    attributes[campo["destino"]] = field.defaultValue 
                                    ToolboxLogger.debug("---->ins - {} = {}".format(campo["destino"], field.defaultValue))  
                                else:
                                    attributes[campo["destino"]] = None
                                    ToolboxLogger.debug("---->ins - {} = {}".format(campo["destino"], 'null'))  


            resultado = self.destino_da.add(table, attributes)
            ToolboxLogger.debug("resultado = {}".format(resultado))  
            if resultado:
                guid = resultado["globalId"] if "globalId" in resultado else '{}'
                self.actualizarEstadistica(table.properties.name, "C", guid)

            return resultado
        except Exception as e:
            self.actualizarEstadistica(table.properties.name, "E")
            ToolboxLogger.info("Tabla: {}, ERROR: ---->{}".format(table.properties.name, e))

        return None

    @ToolboxLogger.log_method
    def actualizarRegistro(self, table, campos_busqueda, campos, registro_origen, valores_pk, operadores_pk = None, conectores_pk = None, usar_geometria = False):
        ToolboxLogger.debug("---->campos_busqueda: {}".format(campos_busqueda))
        ToolboxLogger.debug("---->valores_pk: {}".format(valores_pk))
        attributesRemove=[]
        where = DataAccess.getWhereClause(campos_busqueda, valores_pk, operadores_pk, conectores_pk)
        query = self.destino_da.query(table, ["*"], where, return_geometry = usar_geometria)
        count = len(query) if query != None else 0
        try:
            if(count > 0) :
                for feature in query:
                    attributes = {}
                    if "geometry" in feature and "geometry" in registro_origen:
                        # ToolboxLogger.debug("feature['geometry'] = {}".format(feature["geometry"]))
                        # ToolboxLogger.debug("registro_origen['geometry'] = {}".format(registro_origen["geometry"]))
                        if feature["geometry"] != registro_origen["geometry"] :
                            attributes["geometry"] = registro_origen["geometry"]
                    
                    if campos:
                        for campo in campos:
                            if "origen" in campo and campo["origen"] in registro_origen:
                                if campo["origen"] != "ND":
                                    valor = registro_origen[campo["origen"]]
                                elif "destino" in campo:
                                    if campo["origen"] == "ND" and campo["destino"].upper()=="PARENTGLOBALID":
                                        valor = self.parentGlobalId
                                    elif "valor_predeterminado" in campo :
                                        valor = campo["valor_predeterminado"]  
                            elif "valor" in campo :
                                valor = campo["valor"]
                            else :
                                valor = None

                            if valor == None and "valor_predeterminado" in campo:
                                valor = campo["valor_predeterminado"]    

                            if "esGuid" in campo and valor != None:
                                valor = "{}".format(valor)
                                
                            key  = [x for x in feature if x.lower() == campo["destino"].lower()]
                            valor_destino = feature[key[0]] if key != [] else None
                            
                            if valor != None and valor != valor_destino :
                                if str(valor).strip() != '':
                                    fieldToFill = [x for x in table.properties.fields if x.name.upper() == campo["destino"].upper()]
                                    if len(fieldToFill)  > 0 :
                                        if fieldToFill[0].type == 'esriFieldTypeDate' :
                                            attributes[campo["destino"]] = int(valor)  
                                        else:
                                            attributes[campo["destino"]] = str(valor).strip()
                                else :
                                    attributes[campo["destino"]] = None
                                    attributesRemove.append(campo["destino"])
                                ToolboxLogger.debug("---->upd - {} = {}".format(campo["destino"], valor))

                        if len(attributes) > 0 :
                            attributes[table.properties.objectIdField] = feature[table.properties.objectIdField]
                            attributes[table.properties.globalIdField] = feature[table.properties.globalIdField]
                            
                            resultado = self.destino_da.update(table, attributes)
                            ToolboxLogger.debug("resultado = {}".format(resultado))
                            if resultado:
                                guid = resultado["globalId"] if "globalId" in resultado else '{}'
                                self.actualizarEstadistica(table.properties.name, "U", guid)

                            return resultado
                            
                    self.actualizarEstadistica(table.properties.name, "NU", feature[table.properties.globalIdField])
                    return attributes

        except Exception as e:
            if feature:
                self.actualizarEstadistica(table.properties.name, "E", feature[table.properties.globalIdField])
            else :
                self.actualizarEstadistica(table.properties.name, "E", {})

            ToolboxLogger.info("Tabla: {}, ERROR: ---->{}".format(table.properties.name, e))
        return None 

    @ToolboxLogger.log_method
    def actualizarMapeado(self, nombre_tabla, campo_busqueda, campos, registro_origen, valor_relacion, operadores = None, conectores = None, tipo_operacion = "U", usar_geometria = False):
        
        if nombre_tabla == "[main_feature]":
            tabla = self.destino_da.getTable(self.claseDestinoPrincipal)
        else:
            tabla = self.destino_da.getTable(nombre_tabla)

        r = None
        if tipo_operacion == "C":
            r = self.crearRegistro(
                tabla, campo_busqueda, campos, registro_origen, valor_relacion, usar_geometria
            )
        elif tipo_operacion == "U":
            r = self.actualizarRegistro(
                tabla, campo_busqueda, campos, registro_origen, valor_relacion, operadores, conectores, usar_geometria
            )
        elif tipo_operacion == "CU":
            r = self.actualizarRegistro(
                tabla, campo_busqueda, campos, registro_origen, valor_relacion, operadores, conectores, usar_geometria
            )
            if r == None:
                r = self.crearRegistro(
                    tabla, campo_busqueda, campos, registro_origen, valor_relacion, usar_geometria
                )
        return r

    @ToolboxLogger.log_method
    def actualizarRelacionesOrigen(self, relacion_origen,
                                    registro_origen,
                                    valor_pk_origen,
                                    valor_pk_destino_padre):
        #ToolboxLogger.debug("relacion_origen: {}".format(relacion_origen))
        #ToolboxLogger.debug("registro_origen: {}".format(registro_origen))
        ToolboxLogger.debug("valor_pk_origen: {}".format(valor_pk_origen))
        ToolboxLogger.debug("valor_pk_destino_padre: {}".format(valor_pk_destino_padre))

        tbl_origen_hija = relacion_origen["origenHija"]
        tbl_destino = relacion_origen["destinoHija"]
        campos_destino = relacion_origen["campos"]
        tipoOperacion = relacion_origen["tipoOperacion"]
        llaveRelacionOrigen = relacion_origen["llaveRelacionOrigen"]
        llaveRelacionDestino = relacion_origen["llaveRelacionDestino"]
        llaveForaneaOrigen = relacion_origen["llaveForaneaOrigen"]
        esllaveForaneaOrigenGuid = relacion_origen["esllaveForaneaOrigenGuid"]
        llavePrimariaDestino = relacion_origen["llavePrimariaDestino"]
        llavePrimariaOrigen = relacion_origen["llavePrimariaOrigen"]
        detalleRelacion = relacion_origen["detalleRelacion"] if "detalleRelacion" in relacion_origen else None
        if detalleRelacion :
            cardinalidad = detalleRelacion["cardinalidad"] 
            llaveForaneaHija = detalleRelacion["llaveForaneaHija"]
            relationshipClass = detalleRelacion["relationshipClass"] if "relationshipClass" in detalleRelacion else None
            llaveForaneaPadre = detalleRelacion["llaveForaneaPadre"] if "llaveForaneaPadre" in detalleRelacion else None

        condicion = relacion_origen["condicion"] if "condicion" in relacion_origen else None
        idRelacion = relacion_origen["idRelacion"] if "idRelacion" in relacion_origen else None
        filtro = relacion_origen["filtroDestino"] if "filtroDestino" in relacion_origen else None
        filtroOrigen = relacion_origen["filtroOrigen"] if "filtroOrigen" in relacion_origen else None
        conectorFiltroDestino = relacion_origen["conectorFiltroDestino"] if "conectorFiltroDestino" in relacion_origen else None
        conectorFiltroOrigen = relacion_origen["conectorFiltroOrigen"] if "conectorFiltroOrigen" in relacion_origen else None
        usarGeometria = relacion_origen["usarGeometria"] if "usarGeometria" in relacion_origen else False

        if tbl_destino == "[main_feature]":
            tbl_destino = self.claseDestinoPrincipal

        ToolboxLogger.debug("Origen: {}".format(tbl_origen_hija))
        ToolboxLogger.debug("Destino: {}".format(tbl_destino))
        ToolboxLogger.debug("Relación '{}'".format(cardinalidad))
        ToolboxLogger.debug("llaveForaneaHija: {}".format(llaveForaneaHija))
        
        if filtroOrigen:
            ToolboxLogger.debug("Filtro Origen: {}".format(filtroOrigen))
        if filtro:
            ToolboxLogger.debug("Filtro Destino: {}".format(filtro))

        self.actualizarEstadistica(tbl_destino)

        if esllaveForaneaOrigenGuid:
            valor_pk_origen = "{}".format(valor_pk_origen)

        # Esta consulta se hace para obtener los registros de la tabla origen hija relacionados
        # con la tabla origen padre. En condiciones normales responden a la consulta con la claúsula
        # where [llaveForaneaOrigen] = valor_pk_origen más los filtros aplicados a los campos 
        # de la tabla origen hija 

        operador = None
        if llaveForaneaOrigen != "ND":
            llave_Relacion = [llaveForaneaOrigen]
            valor_Relacion = [registro_origen[relacion_origen["campoBusquedaOrigen"]] if "campoBusquedaOrigen" in relacion_origen else valor_pk_origen]
        else:
            llave_Relacion = [llaveRelacionOrigen]
            valor_Relacion = [registro_origen[relacion_origen["campoBusquedaOrigen"]]]

        if filtroOrigen:
            if "operador" in filtroOrigen:
                operador = [None, filtroOrigen["operador"]]
            llave = llave_Relacion + [filtroOrigen["campo"]]
            valor = valor_Relacion + [filtroOrigen["valor"]]
        else:
            llave = llave_Relacion
            valor = valor_Relacion

        where = DataAccess.getWhereClause(llave, valor, operador)
        registros_origen_hija = self.fuente_da.query(tbl_origen_hija, ["*"], where, return_geometry= usarGeometria)

        origen_hija_i = 0
        origen_hija_n = len(registros_origen_hija)

        for registro_origen_hija in registros_origen_hija:

            origen_hija_i += 1
            ToolboxLogger.debug("registro_origen_hija: '{}' {} de {}".format(tbl_origen_hija, origen_hija_i, origen_hija_n))

            ##Procesamiento registros dependientes
            if not self.cumpleCondicion(condicion, registro_origen_hija):
                ToolboxLogger.debug("No cumple condición")
                return

            valor_rk_origen_hija = registro_origen_hija[llaveRelacionOrigen] if llaveRelacionOrigen in registro_origen_hija else ""
            ToolboxLogger.debug("valor_rk_origen_hija: '{}' = {}".format(llaveRelacionOrigen, valor_rk_origen_hija))

            if filtro:
                if not "valor" in filtro and "campo" in filtro:
                    if isinstance(filtro["campo"], str) :
                        campos_filtro = []
                        campos_filtro.append(filtro["campo"])
                    else :
                        campos_filtro = filtro["campo"]
                    valores_filtro = []

                    for campo in campos_filtro :
                        campo_mapeo = [x for x in campos_destino if x["destino"].lower() == campo.lower()]
                        valor_destino = registro_origen_hija[campo_mapeo[0]["origen"]] if campo_mapeo else None
                        if valor_destino:
                            valores_filtro.append(valor_destino)

                    ToolboxLogger.debug("filtro: {}".format(filtro))
                else :
                    valores_filtro = filtro["valor"]

            if cardinalidad == "M-M":
                if "tipoOperacion" in relacion_origen["detalleRelacion"]:
                    tipoOperacionRelacion = relacion_origen["detalleRelacion"]["tipoOperacion"]
                else:
                    tipoOperacionRelacion = tipoOperacion

                llave_relacion = [llaveRelacionDestino]
                valor_relacion = [registro_origen_hija[llaveRelacionOrigen]]

                if filtro:
                    llave_relacion = llave_relacion + [filtro["campo"]]
                    valor_relacion = valor_relacion + [valores_filtro]
                
                conector_relacion = [conectorFiltroDestino] if conectorFiltroDestino else None

                ToolboxLogger.debug("Tabla Principal")
                ##Cuando la relacion es M-M y se inserta un registro en una de las tablas de rompimiento se hace necesario
                ##obtener el GlobalId del registro insertado para poderlo insertar en las tablas de rompimiento.
                ##Por eso se recibe el resultado de la operacion de edición o inserción para usarlo posteriormente como 
                ##llave en el filtro que va a permitir el registro en la tabla de rompimiento
                resultadoOperacion = self.actualizarMapeado(
                                    tbl_destino,
                                    llave_relacion,
                                    campos_destino,
                                    registro_origen_hija,
                                    valor_relacion,
                                    conectores= conector_relacion,
                                    tipo_operacion = tipoOperacion, 
                                    usar_geometria = usarGeometria
                )
                if resultadoOperacion != {}:
                    guid = self.verificarRegistroOrigen(
                        tbl_origen_hija, 
                        llaveRelacionOrigen, 
                        registro_origen_hija, 
                        resultadoOperacion)
                    valor_pk_destino_hija = guid
                    valor_relacion[0] = guid
                else :
                    valor_pk_destino_hija = registro_origen_hija[llaveRelacionOrigen]

                llave_relacion_mm = [llaveForaneaPadre, llaveForaneaHija]
                valor_relacion_mm = [valor_pk_destino_padre, valor_pk_destino_hija]
                where = DataAccess.getWhereClause(llave_relacion_mm, valor_relacion_mm)
                registros_relacion = self.destino_da.query(relationshipClass, [llaveForaneaPadre, llaveForaneaHija], where)

                if tipoOperacionRelacion == "U" and len(registros_relacion) < 1:
                    valor_relacion_mm = [valor_pk_destino_padre, "NULL"]
                    operadores_relacion = [None, "IS"]
                    where = DataAccess.getWhereClause(llave_relacion_mm, valor_relacion_mm, operadores_relacion)
                    registros_relacion = self.destino_da.query(relationshipClass, [llavePrimariaDestino, llaveForaneaPadre, llaveForaneaHija], where)
                    llave_relacion_mm = llavePrimariaDestino
                    campos_relacion_mm = [{"origen": "ND", 
                                           "destino" : llaveForaneaHija, 
                                           "valor_predeterminado" :  valor_pk_destino_hija}]

                    relacion_i = 0
                    relacion_n = len(registros_relacion)
                    for registro_relacion in registros_relacion :
                        valor_relacion_mm = registro_relacion[llavePrimariaDestino]

                        relacion_i += 1
                        ToolboxLogger.debug("Relación: {} de {}".format(relacion_i, relacion_n))

                        ToolboxLogger.debug(
                            "valor_relacion_mm: {} = '{}'".format(
                                llave_relacion_mm,
                                valor_relacion_mm)
                        )
                        ToolboxLogger.debug("Tabla Relación")
                        resultadoOperacion = self.actualizarMapeado(
                            relationshipClass,
                            llave_relacion_mm,
                            campos_relacion_mm,
                            registro_relacion,
                            valor_relacion_mm, 
                            tipo_operacion = tipoOperacionRelacion, 
                            usar_geometria = usarGeometria
                        )
                else :
                    campos_relacion_mm = None
                    relacion_i = 0
                    relacion_n = len(registros_relacion)

                    if relacion_n > 0:
                        for registro_relacion in registros_relacion :
                            relacion_i += 1
                            ToolboxLogger.debug("Relación: {} de {}".format(relacion_i, relacion_n))

                            ToolboxLogger.debug(
                                "valor_pk_destino_padre: {}".format(
                                    valor_pk_destino_padre)
                            )
                            ToolboxLogger.debug(
                                "valor_pk_destino_hija: {}".format(
                                    valor_pk_destino_hija)
                            )
                            ToolboxLogger.debug("Tabla Relación")
                            resultadoOperacion = self.actualizarMapeado(
                                relationshipClass,
                                llave_relacion_mm,
                                campos_relacion_mm,
                                registro_relacion,
                                valor_relacion_mm, 
                                tipo_operacion = tipoOperacionRelacion, 
                                usar_geometria = usarGeometria
                            )
                    else :
                        ToolboxLogger.debug(
                            "valor_pk_destino_padre: {}".format(
                                valor_pk_destino_padre)
                        )
                        ToolboxLogger.debug(
                            "valor_pk_destino_hija: {}".format(
                                valor_pk_destino_hija)
                        )
                        ToolboxLogger.debug("Tabla Relación")
                        resultadoOperacion = self.actualizarMapeado(
                            relationshipClass,
                            llave_relacion_mm,
                            campos_relacion_mm,
                            None,
                            valor_relacion_mm, 
                            tipo_operacion = tipoOperacionRelacion, 
                            usar_geometria = usarGeometria
                        )

            elif cardinalidad == "1-M":

                # Para cada registro de la tabla origen hija se efectúa actualizarMapeado, que actualiza
                # los registros de la tabla destino hija en caso de que existan o los crea en caso de que no
                # existan. Internamente la operación hace una consulta en la tabla destino hija para determinar
                # si el registro existe y en principio la consulta obtiene el registro de la tabla destino hija
                # que esté relacionado con el registro de correspondiente de la tabla destino padre mediante
                # la llaveForaneaHija que debe ser igual a la llaveForaneaDestino si existe, pero como esta consulta
                # puede arrojar varios registros y la operacion actualizarMapeo actua sobre un solo registro se debe 
                # agregar otra llave para hacer el resultado unico, esto se logra relacionando adicionamente el registro
                # actual de la tabla origen hija con la tabla destino hija mediante el par de llaves llaveRelacionOrigen
                # y llaveRelacionDestino. En condicones normales esta consulta debería tener una claúsula where del tipo
                # "llaveRelacionDestino = valor_rk_origen_hija AND llaveForaneHija = valor_pk_destino_padre" más 
                # los filtros que se agreguen a los campos de la tabla destino hija.

                if valor_rk_origen_hija != '' :
                    llave_relacion = [llaveRelacionDestino, llaveForaneaHija]
                    llaveAlternaOrigen = detalleRelacion["campoLlaveRegOrigen"] if "campoLlaveRegOrigen" in detalleRelacion else None
                    if llaveAlternaOrigen :
                        # self.parentGlobalId = valor_pk_destino_padre
                        valor_relacion = [registro_origen_hija[llaveAlternaOrigen], self.parentGlobalId]
                    else:
                        valor_relacion = [valor_rk_origen_hija, valor_pk_destino_padre]
                else :
                    llave_relacion = [llaveForaneaHija]
                    valor_relacion = [valor_pk_destino_padre]

                if filtro:
                    if isinstance(filtro, list) :
                        llave = llave_relacion + filtro["campo"]
                        valor = valor_relacion + filtro["valor"]
                    else :
                        llave = llave_relacion + [filtro["campo"]]
                        valor = valor_relacion + [filtro["valor"]]
                else :
                    llave = llave_relacion
                    valor = valor_relacion
                        
                ToolboxLogger.debug("Tabla Principal")
                resultadoOperacion = self.actualizarMapeado(
                    tbl_destino,
                    llave,
                    campos_destino,
                    registro_origen_hija,
                    valor,
                    tipo_operacion = tipoOperacion, 
                    usar_geometria = usarGeometria
                )
                if resultadoOperacion != {}:
                    guid = self.verificarRegistroOrigen(
                        tbl_origen_hija, 
                        llaveRelacionOrigen, 
                        registro_origen_hija, 
                        resultadoOperacion)
                    valor_relacion[0] = guid if llave_relacion[0] == llavePrimariaDestino else valor_relacion[0]

            where = DataAccess.getWhereClause(llave_relacion, valor_relacion)
            registros_resultado = self.destino_da.query(tbl_destino, [llaveRelacionDestino, llavePrimariaDestino], where)
            ToolboxLogger.debug("registros_resultado--> {}".format(registros_resultado))

            # Procesamiento Recursivo
            relacionesOrigenHija = relacion_origen["relacionesOrigen"] if "relacionesOrigen" in relacion_origen else []
            if relacionesOrigenHija:
                ToolboxLogger.debug("Procesar Relaciones Origen")
                ToolboxLogger.debug("llave_relacion: '{}'".format(llave_relacion))
                ToolboxLogger.debug("valor_relacion: '{}'".format(valor_relacion))

                if isinstance(llave_relacion, str) and llave_relacion.lower() == 'parentglobalid':
                    self.parentGlobalId = valor_relacion

                where = DataAccess.getWhereClause(llave_relacion, valor_relacion)
                registros_destino_hija = self.destino_da.query(tbl_destino, [llaveRelacionDestino, llavePrimariaDestino], where)

                valor_pk_origen_padre = registro_origen[llavePrimariaOrigen] if llavePrimariaOrigen in registro_origen else ""
                ToolboxLogger.debug("valor_pk_origen_padre: '{}' = '{}'".format(llavePrimariaOrigen, valor_pk_origen_padre))

                valor_fk_origen_hija = registro_origen_hija[llaveForaneaOrigen] if llaveForaneaOrigen in registro_origen_hija else ""
                ToolboxLogger.debug("valor_fk_origen_hija : '{}' = '{}'".format(llaveForaneaOrigen, valor_fk_origen_hija))

                valor_pk_origen_hija = registro_origen_hija[llavePrimariaOrigen] if llavePrimariaOrigen in registro_origen_hija else ""
                ToolboxLogger.debug("valor_pk_origen_hija : '{}' = '{}'".format(llavePrimariaOrigen, valor_pk_origen_hija))

                destino_hija_i = 0
                destino_hija_n = len(registros_destino_hija)

                for registro_destino_hija in registros_destino_hija :
                    destino_hija_i += 1
                    ToolboxLogger.debug("Registro Origen Hija: '{}' {} de {}".format(tbl_destino, destino_hija_i, destino_hija_n))

                    valor_pk_destino = registro_destino_hija[llavePrimariaDestino]

                    ToolboxLogger.debug(
                        "valor_pk_destino: '{}' = '{}'".format(llavePrimariaDestino, valor_pk_destino))
                    ToolboxLogger.debug("****Procesamiento Recursivo****")

                    relacion_origen_i = 0
                    relacion_origen_n = len(relacionesOrigenHija)

                    for relacion in relacionesOrigenHija:
                        relacion_origen_i += 1
                        ToolboxLogger.debug("Relacion Origen '{}' {} de {}".format(tbl_destino, relacion_origen_i, relacion_origen_n))

                        self.actualizarRelacionesOrigen(
                            relacion, registro_origen_hija, valor_pk_origen_hija, valor_pk_destino)

    @ToolboxLogger.log_method
    def actualizarRelacionesDestino(self, relacion_destino, registro_origen, valor_pk_destino):

        tbl_destino = relacion_destino["destino"]
        llaveForaneaDestino = relacion_destino["llaveForaneaDestino"]
        llavePrimariaDestino = relacion_destino["llavePrimariaDestino"]
        llavePrimariaOrigen = relacion_destino["llavePrimariaOrigen"]
        llaveEsGuid = relacion_destino["llaveEsGuid"]

        campos = relacion_destino["campos"]
        tipoOperacion = relacion_destino["tipoOperacion"]
        condicion = relacion_destino["condicion"] if "condicion" in relacion_destino else None
        idRelacion = relacion_destino["idRelacion"] if "idRelacion" in relacion_destino else None
        usarGeometria = relacion_destino["usarGeometria"] if "usarGeometria" in relacion_destino else False

        if tbl_destino == "[main_feature]":
            tbl_destino = self.claseDestinoPrincipal
        self.actualizarEstadistica(tbl_destino)

        ToolboxLogger.debug("Destino: {}".format(tbl_destino))

        if not self.cumpleCondicion(condicion, registro_origen):
            ToolboxLogger.debug("No cumple condición")
            return

        resultadoOperacion = self.actualizarMapeado(
            tbl_destino,
            llaveForaneaDestino,
            campos,
            registro_origen,
            valor_pk_destino,
            tipo_operacion = tipoOperacion, 
            usar_geometria = usarGeometria
        )

        # Para obtener el valor de la llave Primaria Destino
        where = DataAccess.getWhereClause(
            llaveForaneaDestino, valor_pk_destino
        )
        registros_destino = self.destino_da.query(tbl_destino, llavePrimariaDestino, where)

        if len(registros_destino) if registros_destino != None else 0 == 1:
            valor_pk_destino = registros_destino[0][llavePrimariaDestino]
            if llaveEsGuid:
                valor_pk_destino = "{}".format(valor_pk_destino)

            # Actualizar Relaciones Origen
            valor_pk_origen = registro_origen[llavePrimariaOrigen]

            relacionesOrigenHija = relacion_destino["relacionesOrigen"] if "relacionesOrigen" in relacion_destino else [
            ]
            if relacionesOrigenHija:
                ToolboxLogger.debug("Procesar Relaciones Origen")

                ToolboxLogger.debug("****Procesamiento Recursivo****")
                for relacion in relacionesOrigenHija:
                    self.actualizarRelacionesOrigen(
                        relacion, registro_origen, valor_pk_origen, valor_pk_destino)

        elif len(registros_destino) > 1:
            ToolboxLogger.debug(
                "Error de Integridad Refencial: {}".format(tbl_destino)
            )

    @ToolboxLogger.log_method
    def procesarRelacionesOrigen(self, mapeo, registro_origen, valor_pk_origen, valor_pk_destino_padre):
        relacionesOrigen = mapeo["relacionesOrigen"]

        for relacion in relacionesOrigen:
            self.actualizarRelacionesOrigen(
                relacion, registro_origen, valor_pk_origen, valor_pk_destino_padre)

    @ToolboxLogger.log_method
    def procesarRelacionesDestino(self, mapeo, registro_origen, valor_pk_destino):
        relaciones_destino = mapeo["relacionesDestino"]

        for relacion_destino in relaciones_destino:
            self.actualizarRelacionesDestino(
                relacion_destino, registro_origen, valor_pk_destino)

    @ToolboxLogger.log_method
    def actualizarRegistroOrigen(self, tabla, registro, llave, valor = None) :
        pass

    @ToolboxLogger.log_method
    def actualizarRegistroDestino(self, tabla, registro, llave, valor = None) :
        pass
    
    def agregarInforme(self) :
        pass

    @ToolboxLogger.log_method
    def verificarRegistroOrigen(self, tabla, llave, registro, resultadoOperacion) :
        if resultadoOperacion :
            valor_relacion = resultadoOperacion["globalId"] if "globalId" in resultadoOperacion else None
            if valor_relacion :
                    self.actualizarRegistroOrigen(tabla, registro, llave, valor_relacion)
        else :
            valor_relacion = None
                    
        return valor_relacion

    @ToolboxLogger.log_method
    def procesarMapeo(self, mapeo, queryList):
        self.estadisticas.timer.initTimer()
        try :
            # Actualizar tabla principal
            origen = mapeo["origen"]
            destino = mapeo["destino"]
            llaveRelacionOrigen = mapeo["llaveRelacionOrigen"]
            llaveRelacionDestino = mapeo["llaveRelacionDestino"]
            llavePrimariaDestino = mapeo["llavePrimariaDestino"]
            llavePrimariaOrigen = mapeo["llavePrimariaOrigen"]
            llaveEsGuid = mapeo["llaveEsGuid"]
            campos = mapeo["campos"]
            tipoOperacion = mapeo["tipoOperacion"]
            usarGeometria = mapeo["usarGeometria"] if "usarGeometria" in mapeo else False
            pattern = "{:<45} {:>3} creados {:>3} actualizados {:>3} no actualizados {:>3} errores {:>3} registros {:>3} operaciones"
            ancho = len(pattern) + 29

            if origen == "[main_feature]":
                origen = self.obtenerTablaPrincipal()
            if destino == "[main_feature]":
                destino = self.claseDestinoPrincipal

            ToolboxLogger.info("origen: '{}'".format(origen))
            ToolboxLogger.info("destino: '{}'".format(destino))
            ToolboxLogger.debug("llaveRelacionOrigen: '{}'".format(llaveRelacionOrigen))

            campos_origen = []
            campos_origen.append(llaveRelacionOrigen)

            for campo in campos:
                campos_origen.append(campo["origen"])

            tbl_origen = origen
            tbl_destino = destino

            filtroOrigen = mapeo["filtroOrigen"] if "filtroOrigen" in mapeo else None
            if filtroOrigen :
                queryList.addQuery(QueryItem(filtroOrigen["campo"], filtroOrigen["valor"]))

            where = queryList.getWhereClause()
            datos_origen = self.fuente_da.query(tbl_origen, ["*"], where, return_geometry = usarGeometria)
            ToolboxLogger.info("Origen: {}, {} registros encontrados".format(tbl_origen, len(datos_origen)))
            ToolboxLogger.debug("filtro: {}".format(where))

            procesados = 0
            for registro_origen in datos_origen:
                procesados += 1
                ToolboxLogger.info("Procesando Encuesta: {}".format(procesados))
                ToolboxLogger.debug("-".ljust(ancho, "-"))

                self.estadisticas.agregarEstadistica()
                self.estadisticas.agregarEncuesta(registro_origen[llavePrimariaOrigen])

                valor_primaria_origen = registro_origen[llavePrimariaOrigen]
                valor_relacion_origen = registro_origen[llaveRelacionOrigen]
                ToolboxLogger.debug("valor_primaria_origen: '{}' = '{}'".format(llavePrimariaOrigen, valor_primaria_origen))
                ToolboxLogger.debug("valor_relacion_origen: '{}' = '{}'".format(llaveRelacionOrigen, valor_relacion_origen))

                resultadoOperacion = self.actualizarMapeado(
                    tbl_destino,
                    llaveRelacionDestino,
                    campos,
                    registro_origen,
                    valor_relacion_origen,
                    tipo_operacion = tipoOperacion, 
                    usar_geometria = usarGeometria
                )
                if resultadoOperacion != {}:
                    guid = self.verificarRegistroOrigen(
                        tbl_origen, 
                        llaveRelacionOrigen, 
                        registro_origen, 
                        resultadoOperacion)
                    valor_relacion_origen = guid if llaveRelacionDestino == llavePrimariaDestino else valor_relacion_origen


                # Para obtener el valor de la llave Primaria Destino
                where = DataAccess.getWhereClause(llaveRelacionDestino, valor_relacion_origen)
                registros_destino = self.destino_da.query(tbl_destino, ["*"], where)

                # Las llaves RelacionOrigen y RelacionDestino deberían ser del tipo GUID y deberían ser únicas
                # por lo tanto si la consulta trae más de un registro se presenta un error de integridad referencial
                # entre el origen y el destino es una relación 0..1

                if len(registros_destino) if registros_destino != None else 0 == 1:
                    valor_pk_destino = registros_destino[0][llavePrimariaDestino]
                    if llaveEsGuid:
                        valor_pk_destino = "{}".format(valor_pk_destino)

                    #Actualizar Relaciones Destino
                    ToolboxLogger.debug("Procesando Relaciones Destino")
                    ToolboxLogger.debug("valor_pk_destino: {}".format(valor_pk_destino))
                    self.procesarRelacionesDestino(mapeo, registro_origen, valor_pk_destino)

                    # Actualizar Relaciones Origen
                    ToolboxLogger.debug("Procesando Relaciones Origen")
                    valor_pk_origen = registro_origen[llavePrimariaOrigen]
                    ToolboxLogger.debug("valor_pk_origen: {}".format(valor_pk_origen))
                    self.parentGlobalId = valor_pk_destino
                    self.procesarRelacionesOrigen(mapeo, registro_origen, valor_pk_origen, valor_pk_destino)

                elif len(registros_destino) > 1:
                    ToolboxLogger.info("Error de Integridad Refencial: {}".format(destino))
                    self.actualizarEstadistica(tbl_destino, "E")
                
                self.actualizarRegistroOrigen(tbl_origen, registro_origen, llavePrimariaOrigen)
                self.actualizarRegistroDestino(tbl_destino, registros_destino[0], llavePrimariaDestino)
            ToolboxLogger.info("")
        except Exception as e:
            ToolboxLogger.info("!".ljust(ancho, "*"))
            ToolboxLogger.info("Error: {}".format(e))
            ToolboxLogger.info("!".ljust(ancho, "*"))
        finally :
            self.estadisticas.timer.stopTimer()

            ToolboxLogger.info("-".ljust(ancho, "-"))
            for tabla in self.estadisticas.tablasEstadisticas:
                ToolboxLogger.info(pattern.format(tabla,
                    self.estadisticas.obtenerTotalEstadisticasTabla(tabla, "C"),
                    self.estadisticas.obtenerTotalEstadisticasTabla(tabla, "U"),
                    self.estadisticas.obtenerTotalEstadisticasTabla(tabla, "NU"),
                    self.estadisticas.obtenerTotalEstadisticasTabla(tabla, "E"),
                    self.estadisticas.obtenerTotalRegistrosTabla(tabla),
                    self.estadisticas.obtenerTotalOperacionesTabla(tabla)))

            ToolboxLogger.info("-".ljust(ancho, "-"))
            ToolboxLogger.info("Total Tablas         : {}".format(self.estadisticas.obtenerTotalTablas()))
            ToolboxLogger.info("Total Encuestas      : {}".format(self.estadisticas.obtenerNumeroEstadisticas()))
            ToolboxLogger.info("Total Creados        : {}".format(self.estadisticas.obtenerTotalEstadisticas("C")))
            ToolboxLogger.info("Total Actualizados   : {}".format(self.estadisticas.obtenerTotalEstadisticas("U")))
            ToolboxLogger.info("Total No Actualizados: {}".format(self.estadisticas.obtenerTotalEstadisticas("NU")))
            ToolboxLogger.info("Total Errores        : {}".format(self.estadisticas.obtenerTotalEstadisticas("E")))
            ToolboxLogger.info("Total Registros      : {}".format(self.estadisticas.obtenerTotalRegistros()))
            ToolboxLogger.info("Total Operaciones    : {}".format(self.estadisticas.obtenerTotalOperaciones()))
            ToolboxLogger.info("Tiempo Total         : {}".format(self.estadisticas.timer.timeSpan))
          
            self.agregarInforme()

            if self.estadisticas.obtenerTotalRegistros() > 0:
                ToolboxLogger.info("Tiempo por encuesta    : {}".format(self.estadisticas.duracionPorEstadistica()))
                ToolboxLogger.info("Tiempo por registro    : {}".format(self.estadisticas.duracionPorRegistro()))
                ToolboxLogger.info("Tiempo por operación   : {}".format(self.estadisticas.duracionPorOperacion()))
                ToolboxLogger.info("Encuesta por minuto    : {}".format(self.estadisticas.estadisticasPorMinuto()))
                ToolboxLogger.info("Registros por minuto   : {}".format(self.estadisticas.registrosPorMinuto()))
                ToolboxLogger.info("Operaciones por minuto : {}".format(self.estadisticas.operacionesPorMinuto()))
            else :
                ToolboxLogger.info("Tiempo por registro  : {}".format(0))
            ToolboxLogger.info("-".ljust(ancho, "-"))

            return  len(datos_origen), self.estadisticas.obtenerTotalRegistros(), self.estadisticas.obtenerTotalEstadisticas("E")




# -*- coding: utf-8 -*-

from datetime import datetime
from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.Utiles import Configuracion
from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess


class ReplicarServicio : 

    def __init__(self, rutaConfiguracion):

        self.rutaConfiguracion = rutaConfiguracion
        self.config = Configuracion(rutaConfiguracion)

     
    @ToolboxLogger.log_method
    def Ejecutar(self, portal, usuario, clave, servicioFuente, numeroServiciosDestino) :
        try: 
            ToolboxLogger.info("Portal: {}".format(portal))
            ToolboxLogger.info("Servicio Origen: {}".format(servicioFuente))
            ToolboxLogger.info("Número Servicios Destino: {}".format(numeroServiciosDestino))
            ToolboxLogger.info("Usuario: {}".format(usuario))

            total_servicios = 0
            total_errores = 0

            fuente_da = ArcGISPythonApiDataAccess(portal, usuario, clave)
            fuente_da.setFeatureService(servicioFuente)
            elementoCopiar = fuente_da.searchElement(servicioFuente, 'Form')
            ##elements_to_copy = fuente_da.getElementsInFolder("Survey-FUCM30303")
            prefijoNombreServicio = self.config.getConfigKey("prefijoNombreServicios")
            for x in range(numeroServiciosDestino): 
                nombre_servicio = "{}_{}". format(prefijoNombreServicio, x)
                servicioBuscar = fuente_da.searchElement(nombre_servicio, 'Feature Service')
                if (len(servicioBuscar) == 0):
                    fuente_da.copyFeatureService(nombre_servicio)
                    ToolboxLogger.info("Servicio: {} creado.".format(nombre_servicio))
                
                elementoBuscar = fuente_da.searchElement(nombre_servicio, 'Form')
                if (len(elementoBuscar) == 0):
                    fuente_da.copyElement(elementoCopiar[0], nombre_servicio)
                    ToolboxLogger.info("Elemento: {} creado.".format(nombre_servicio))
                
                total_servicios +=1
                
                ##total_errores += errores
            
            return fuente_da, total_servicios, total_errores

        except Exception as e:
            ToolboxLogger.error("Error: {}".format(e))
        else :
            destino_da.setMode("None")
        finally :
            ToolboxLogger.info("Terminado.")



# -*- coding: utf-8 -*-

from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess
from ImportarSurvey.ImportarDatosInyeccion import ImportarDatosInyeccion
from ImportarSurvey.ProcesarEncuestas import ProcesarEncuestas

class InyectarEncuestas(ProcesarEncuestas) : 

    def __init__ (self, 
        rutaConfiguracion, 
        portal = None, 
        usuario = None, 
        clave = None, 
        servicioFuente = None, 
        servicioDestino = None, 
        versionFuente = None, 
        versionDestino = None, 
        usuarioCampo = None):
        super().__init__(
            rutaConfiguracion, 
            portal, 
            usuario, 
            clave, 
            servicioFuente,  
            servicioDestino, 
            versionFuente = versionFuente,
            versionDestino = versionDestino, 
            usuarioCampo = usuarioCampo)

    @ToolboxLogger.log_method
    def calcularConsultaInicial(self) :
        campos =  ["estadoenc", "estado_sincronizacion"]
        valores = ["cerrada", "sin_sincronizar"]

        tabla = self.fuente_da.getTable(self.servicioFuente)
        f = [x for x in tabla.properties.fields if x.name.lower() == "usuario_portal"]
        if f:
            campos.append("usuario_portal")
            valores.append(self.usuario_campo)

        self.obtenerConsultaInicial(campos, valores)
        
        return

    @ToolboxLogger.log_method
    def limpiarProceso(self):
        if self.destino_da:
            ToolboxLogger.info("stopEditing: '{}'".format(self.destino_da.stopEditing(True)))
            ToolboxLogger.info("stopReading: '{}'".format(self.destino_da.stopReading()))
            self.destino_da.setMode(None)
            ToolboxLogger.info("purgeVersion: '{}'".format(self.destino_da.purgeVersion(self.versionDestino)))
        
        return

    @ToolboxLogger.log_method
    def inicializarProceso(self) :
        try :
            ToolboxLogger.info("Portal: '{}'".format(self.portal))
            ToolboxLogger.info("Servicio Origen: '{}'".format(self.servicioFuente))
            ToolboxLogger.info("Servicio Destino: '{}'".format(self.servicioDestino))
            ToolboxLogger.info("Usuario: '{}'".format(self.usuario))
            ToolboxLogger.info("Nombre Usuario: '{}'".format(self.nombreCompleto))
            ToolboxLogger.info("Usuario Campo: '{}'".format(self.usuario_campo))
            ToolboxLogger.info("Versión Destino: '{}'".format(self.versionDestino))

            self.destino_da = ArcGISPythonApiDataAccess(self.gis)
            self.destino_da.setFeatureService(self.servicioDestino)
            self.destino_da.setVersionManager()

            if not self.versionDestino:
                self.versionDestino = self.crearVersion(self.destino_da, self.servicioFuente)
                ToolboxLogger.info("Se creó la version: '{}'".format(self.versionDestino))
            else :
                ToolboxLogger.info("Versión Destino: '{}'".format(self.versionDestino))

            if self.destino_da.setVersion(self.versionDestino) :
                bloqueado, purgado = self.destino_da.isVersionLocked(self.versionDestino)
                bloqueado = False
                if not bloqueado:
                    self.destino_da.setMode("edit")
                    ToolboxLogger.info("startReading: '{}'".format(self.destino_da.startReading()))
                    ToolboxLogger.info("startEditing: '{}'".format(self.destino_da.startEditing()))

                    self.fuente_da = ArcGISPythonApiDataAccess(self.gis)
                    self.fuente_da.setFeatureService(self.servicioFuente)
                    self.fuente_da.outSpatialReference = self.destino_da.getSpatialReference()
                    self.fuente_da.returnZ = self.destino_da.hasZ
                    
                    ToolboxLogger.debug("Referencia Espacial Destino : '{}'".format(self.fuente_da.outSpatialReference))
                    ToolboxLogger.debug("Tiene Z Destino : '{}'".format( self.fuente_da.returnZ))

                    self.importarDatos = ImportarDatosInyeccion(
                        self.fuente_da, 
                        self.destino_da, 
                        self.servicioFuente, 
                        self.servicioDestino, 
                        self.usuario_campo)
                else :
                    ToolboxLogger.info("Error: Versión Destino está en Uso!")
                    if purgado:
                        ToolboxLogger.info("La versión en uso se purgó. Intente de nuevo.")
            else :
                ToolboxLogger.info("Error: Versión Destino no encontrada!")

        except Exception as e:
            ToolboxLogger.info("Error: {}".format(e))
        
        return
            




import os
from fileinput import filename

from ImportarSurvey.Utiles import FILE_HANDLER, STREAM_HANDLER, JsonFile, ToolboxLogger
from ImportarSurvey.PrecargarEncuestas import PrecargarEncuestas
from ImportarSurvey.InyectarEncuestas import InyectarEncuestas
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess

STATISTICS_FILE = "__estadisticas.json"

class ProcesarEncuestas :

    @staticmethod 
    def BorrarEncuestas(
        portal = None, 
        usuario = None, 
        clave = None, 
        servicioEncuesta = None, 
        debug = False,
        rutaSalida = '', 
        salidaRelativa = False) :

        LOG_FILE = "__logBorrarEncuestasVSCode"
        alias = "LogBorrarEncuestasVSCode"

        folder_path = os.path.dirname(os.path.realpath(__file__))
        log_path = os.path.normpath(os.path.join(folder_path, rutaSalida)) if salidaRelativa else rutaSalida

        ToolboxLogger.initLogger(source=alias, log_path=log_path, log_file=LOG_FILE)
        ToolboxLogger.setDebugLevel() if debug else ToolboxLogger.setInfoLevel()

        ToolboxLogger.info("Iniciando {}".format(alias))
        ToolboxLogger.info("Ruta Salida: {}".format(log_path)) if rutaSalida != '' else None

        fuente_da = ArcGISPythonApiDataAccess(portal, usuario, clave)
        fuente_da.setFeatureService(servicioEncuesta)

        numRegistros = 0

        for tabla in fuente_da.getServiceTables() :
            registros = fuente_da.query(tabla, [tabla.properties.objectIdField])
            fuente_da.delete(tabla, registros)
            numRegistros += len(registros)
            ToolboxLogger.debug("Tabla: {} Borrados: {}".format(tabla.properties.name, len(registros)))

        for capa in fuente_da.getServiceLayers() :
            registros = fuente_da.query(capa, [capa.properties.objectIdField])
            fuente_da.delete(capa, registros)
            numRegistros += len(registros)
            ToolboxLogger.debug("Capa: {} Borrados: {}".format(capa.properties.name, len(registros)))

        ToolboxLogger.info("Total Borrados: {}".format(numRegistros))

    @staticmethod
    def Precargar(alias = '',
        archivoLog = '',
        portal = None, 
        usuario = None, 
        clave = None, 
        usuarioCampo = None,
        servicioFuente = None, 
        versionFuente = None, 
        servicioDestino = None, 
        idsPrecarga = None, 
        debug = False, 
        tipoRegistro = (FILE_HANDLER | STREAM_HANDLER),
        rutaSalida = '', 
        salidaRelativa = False, 
        agregarEstadistica = True) :

        CONFIG_BACKWARD_PATH = "_precarga_geo.json"
        LOG_BACKWARD_FILE = "__logPrecargarEncuestasVsCode" if archivoLog == '' else archivoLog
        alias = "PrecargarEncuestasVSCode" if alias == '' else alias

        folder_path = os.path.dirname(os.path.realpath(__file__))
        config_path = os.path.join(folder_path, CONFIG_BACKWARD_PATH)
        log_path = os.path.normpath(os.path.join(folder_path, rutaSalida)) if salidaRelativa else rutaSalida

        ToolboxLogger.initLogger(source = alias, log_path = log_path, log_file = LOG_BACKWARD_FILE, handler_type = tipoRegistro)
        ToolboxLogger.setDebugLevel() if debug else ToolboxLogger.setInfoLevel()

        encuestas = 0
        registros = 0
        errores = 0
        version = ''

        if servicioFuente and versionFuente and servicioDestino and idsPrecarga:
            ToolboxLogger.info("Iniciando {}".format(alias))
            ToolboxLogger.info("Ruta Salida: {}".format(log_path)) if rutaSalida != '' else None

            preloader = PrecargarEncuestas(config_path, 
                portal, 
                usuario, 
                clave,
                servicioFuente, 
                servicioDestino, 
                usuarioCampo= usuarioCampo,
                versionFuente = versionFuente,
                idsPrecarga = idsPrecarga
            )

            resultado  = preloader.Ejecutar()
            if resultado: 
                encuestas = resultado[0]
                registros = resultado[1]
                errores = resultado[2]
                version = preloader.versionFuente

            ToolboxLogger.info("Versión: {}".format(version))
            ToolboxLogger.info("Encuestas: {}".format(encuestas))
            ToolboxLogger.info("Registros: {}".format(registros))
            ToolboxLogger.info("Errores: {}".format(errores))

            if agregarEstadistica:
                filename = os.path.join(log_path, STATISTICS_FILE)
                estadisticas = JsonFile.readFile(filename)
                estadisticas.append(preloader.registroEjecucion())
                JsonFile.writeFile(filename, estadisticas)

        else:
            if not servicioFuente:
                ToolboxLogger.info("No se definió Servicio Fuente")
            if not versionFuente:
                ToolboxLogger.info("No se definió Versión Fuente")
            if not servicioDestino:
                ToolboxLogger.info("No se definió Servicio Destino")
            if not idsPrecarga:
                ToolboxLogger.info("No se definieron IDs de Precarga")

        return encuestas, registros, errores, version

    @staticmethod
    def Inyectar(alias = '',
        archivoLog = '',
        portal = None, 
        usuario = None, 
        clave = None, 
        usuarioCampo = None,
        servicioFuente = None, 
        servicioDestino = None,
        versionDestino = None, 
        debug = False, 
        tipoRegistro = (FILE_HANDLER | STREAM_HANDLER),
        rutaSalida = '', 
        salidaRelativa = False, 
        agregarEstadistica = True) :

        CONFIG_PATH = "_inyeccion_geo.json"
        LOG_FILE = "__logInyectarEncuestasVSCode" if archivoLog == '' else archivoLog
        alias = "LogInyectarEncuestasVSCode" if alias == '' else alias

        folder_path = os.path.dirname(os.path.realpath(__file__))
        config_path = os.path.join(folder_path, CONFIG_PATH)
        log_path = os.path.normpath(os.path.join(folder_path, rutaSalida)) if salidaRelativa else rutaSalida

        ToolboxLogger.initLogger(source = alias, log_path = log_path, log_file = LOG_FILE, handler_type = tipoRegistro)
        ToolboxLogger.setDebugLevel() if debug else ToolboxLogger.setInfoLevel()

        encuestas = 0
        registros = 0
        errores = 0
        versionFinal = ''

        if servicioFuente and versionDestino and servicioDestino:
            ToolboxLogger.info("Iniciando {}".format(alias))
            ToolboxLogger.info("Ruta Salida: {}".format(log_path)) if rutaSalida != '' else None

            inyector = InyectarEncuestas(
                config_path, 
                portal, 
                usuario, 
                clave, 
                servicioFuente, 
                servicioDestino, 
                usuarioCampo = usuarioCampo,
                versionDestino = versionDestino
            )
            resultado  = inyector.Ejecutar()
            if resultado: 
                encuestas = resultado[0]
                registros = resultado[1]
                errores = resultado[2]
                versionFinal = inyector.versionDestino
            
            ToolboxLogger.info("Version Final: {}".format(versionFinal))
            ToolboxLogger.info("Encuestas: {}".format(encuestas))
            ToolboxLogger.info("Registros: {}".format(registros))
            ToolboxLogger.info("Errores: {}".format(errores))

            if agregarEstadistica:
                filename = os.path.join(log_path, STATISTICS_FILE)
                estadisticas = JsonFile.readFile(filename)

                estadisticas.append(inyector.registroEjecucion())
                JsonFile.writeFile(filename, estadisticas)
        else:
            if not servicioFuente:
                ToolboxLogger.info("No se definió Servicio Fuente")
            if not servicioDestino:
                ToolboxLogger.info("No se definió Servicio Destino")
            if not versionDestino:
                ToolboxLogger.info("No se definió Versión Destino")

        return encuestas, registros, errores, versionFinal




import ipywidgets as widgets

from  IPython.display import display
from ImportarSurvey.ImportarSurvey import ProcesarEncuestas

class Precarga :

    @classmethod
    def Mostrar(cls, 
        portal = None, 
        usuario = None, 
        clave = None, 
        usuarioCampo = None,
        servicioFuente = None, 
        versionFuente = None, 
        servicioDestino = None, 
        idsPrecarga = None, 
        salidaRelativa = False, 
        rutaSalida = '') :

        def btn_eventhandler(obj):
            print(portal_text.value)
            
            ProcesarEncuestas.Precargar( 
                portal = portal_text.value, 
                usuario = usuario_text.value, 
                clave = clave_password.value, 
                servicioFuente = servicio_fuente_text.value, 
                versionFuente = version_fuente_text.value, 
                servicioDestino = servicio_destino_text.value, 
                idsPrecarga = id_textarea.value, 
                salidaRelativa = salidaRelativa,
                rutaSalida = rutaSalida)

        layout = widgets.Layout(
            width = 'max_width'
        )

        style = {'description_width': 'max_width'}

        titulo_html = widgets.HTML(
            value="<h1><b>Precarga de Encuestas Catastrales</b></h1>",
        )        
        display(titulo_html)

        portal_text = widgets.Text(
            value = portal,
            description='Url del Portal:',
            style = style,
            disabled=False)
        display(portal_text)

        usuario_text = widgets.Text(
            value = usuario,
            description='Usuario:',
            style = style,
            disabled=False)
        display(usuario_text)

        clave_password = widgets.Password(
            value = clave,
            description='Clave:',
            style = style,
            disabled=False)
        display(clave_password)

        servicio_fuente_text = widgets.Text(
            value = servicioFuente,
            description='Servicio Fuente:',
            style = style,
            disabled=False)
        display(servicio_fuente_text)

        version_fuente_text = widgets.Text(
            value = versionFuente,
            description='Versión Fuente:',
            style = style,
            disabled=False)
        display(version_fuente_text)

        servicio_destino_text = widgets.Text(
            value = servicioDestino,
            description='Servicio Destino:',
            style = style,
            layout = layout,
            disabled=False)
        display(servicio_destino_text)

        id_textarea = widgets.Textarea(
            value = idsPrecarga,
            description='Números Prediales a cargar:',
            style = style,
            disabled=False)
        display(id_textarea)

        usuario_reconocedor_text = widgets.Text(
            value = usuarioCampo,
            description='Usuario Reconocedor:',
            style = style,
            disabled=False)
        display(usuario_reconocedor_text)

        informe_detallado_checkbox = widgets.Checkbox(
            value = False,
            description='Informe Detallado:',
            disabled=False)
        display(informe_detallado_checkbox)

        ejecutar_button = widgets.Button(description ="Ejecutar")
        display(ejecutar_button)

        ejecutar_button.on_click(btn_eventhandler)
    



# -*- coding: utf-8 -*-

from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.ImportarDatos import ImportarDatos

class ImportarDatosInyeccion(ImportarDatos) :

    def __init__(self, 
        fuente_da, 
        destino_da, 
        claseFuentePrincipal = '', 
        claseDestinoPrincipal = '', 
        usuarioCampo = '') :

        super().__init__(
            fuente_da,
            destino_da,
            claseFuentePrincipal = claseFuentePrincipal,
            claseDestinoPrincipal = claseDestinoPrincipal,
            usuarioCampo = usuarioCampo)

    def agregarInforme(self) :
        ToolboxLogger.info("Total Aceptadas      : {}".format(self.estadisticas.totalEstadoEncuesta("estado_aceptacion", "aceptada")))
        ToolboxLogger.info("Total No Aceptadas   : {}".format(self.estadisticas.totalEstadoEncuesta("estado_aceptacion", "no_aceptada")))
        ToolboxLogger.info("Total Sincronizadas  : {}".format(self.estadisticas.totalEstadoEncuesta("estado_sincronizacion", "sincronizada")))

    @ToolboxLogger.log_method
    def actualizarRegistroOrigen(self, nombre_tabla, registro, llave, valor = None) :
        try :
            if nombre_tabla == self.claseFuentePrincipal and not valor :
                registro["estado_sincronizacion"] = "sincronizada"
                registro["estadoenc"] = None

                if self.estadisticas.obtenerEstadisticasActual("E") < 1 :
                    registro["estado_aceptacion"] = "aceptada"
                else :
                    registro["estado_aceptacion"] = "no_aceptada"

                resultado = self.fuente_da.update(nombre_tabla, registro)
                self.estadisticas.actualizarEncuesta(registro[llave], 
                    {"estado_sincronizacion": registro["estado_sincronizacion"],
                     "estado_aceptacion": registro["estado_aceptacion"],
                     "estadoenc": registro["estadoenc"]})
                ToolboxLogger.debug("Resultado Cambio de Estado: {}".format(resultado))
            else :
                if valor and llave in registro and valor != registro[llave]:
                    valor_anterior = registro[llave]
                    tabla = self.fuente_da.getTable(nombre_tabla)

                    registro_a_actualizar = {}
                    registro_a_actualizar[tabla.properties.globalIdField] = registro[tabla.properties.globalIdField]
                    registro_a_actualizar[tabla.properties.objectIdField] = registro[tabla.properties.objectIdField]
                    registro_a_actualizar[llave] = valor

                    resultado = self.fuente_da.update(tabla, registro_a_actualizar)
                    if resultado != {} :
                        guid = resultado["globalId"] if "globalId" in resultado else '{}'
                        ToolboxLogger.debug("Cambio de llave en '{}' para 'globalId' = '{}'':".format(nombre_tabla, guid))
                        ToolboxLogger.debug("'{}' de '{}' a '{}'".format(llave, valor_anterior, valor))
                    else :
                        ToolboxLogger.debug("No cambio de llave en '{}':".format(tabla))
                        ToolboxLogger.debug("Para '{}' = '{}'".format(llave, valor))
        except Exception as ex:
            ToolboxLogger.info("Error: {}".format(ex))






# -*- coding: utf-8 -*-
import platform
import arcgis

from datetime import datetime
from ImportarSurvey.Utiles import ToolboxLogger, Configuracion
from ImportarSurvey.DataAccess import QueryItem, QueryList
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess

class ProcesarEncuestas : 

    def __init__ (self, 
        rutaConfiguracion, 
        portal = None, 
        usuario = None, 
        clave = None, 
        servicioFuente = None, 
        servicioDestino = None, 
        versionFuente = None, 
        versionDestino = None, 
        usuarioCampo = None) :
       
        self.rutaConfiguracion = rutaConfiguracion
        self.config = Configuracion(rutaConfiguracion)

        self.portal = portal
        self.usuario = usuario
        self.usuario_campo = usuarioCampo if usuarioCampo != None else usuario
        self.clave = clave
        self.servicioFuente = servicioFuente
        self.versionDestino = versionDestino
        self.servicioDestino = servicioDestino
        self.versionDestino = versionDestino
        self.versionFuente = versionFuente

        self.destino_da = None
        self.fuente_da = None
        self.importarDatos = None

    @ToolboxLogger.log_method
    def inicializarProceso(self) :
        
        self.gis = ArcGISPythonApiDataAccess.getGIS(self.portal, self.usuario, self.clave)
        self.portal = self.gis.url
        self.usuario = self.gis.users.me.username
        self.usuario_campo = self.usuario_campo if self.usuario_campo != None else self.usuario
        self.nombreCompleto = self.gis.users.me.fullName
        
        self.plataforma = platform.platform()
        self.system, self.node, self.release, self.version, self.machine, self.processor  = platform.uname()

        self.python_impl = platform.python_implementation()
        self.python_rev = platform.python_revision()
        self.python_ver = platform.python_version()
        self.arcgis_ver = arcgis.__version__

        ToolboxLogger.info("Plataforma: {}".format(self.plataforma))
        ToolboxLogger.info("Ejecutando desde: {}".format(self.node))
        ToolboxLogger.info("Sistema Operativo: {} {} Versión: {}".format(self.system, self.release, self.version))
        ToolboxLogger.info("Tipo máquina: {} Procesador: {}".format(self.machine, self.processor))

        ToolboxLogger.info("Python ver: {} rev: {} impl: {}".format(self.python_ver, self.python_rev, self.python_impl))
        ToolboxLogger.info("arcgis ver: {}".format(self.arcgis_ver))
        return

    @ToolboxLogger.log_method
    def limpiarProceso(self):
        return

    @ToolboxLogger.log_method
    def calcularConsultaInicial(self) :
        return 

    @ToolboxLogger.log_method
    def crearVersion(self, da, nombre_formulario) :
        fecha_hora = datetime.now()
        nombre_version = "{}_{}".format(nombre_formulario, fecha_hora.strftime("%Y%m%d%H%M%S"))
        descripcion = "Carga Formulario Survey {}, {}".format(nombre_formulario, fecha_hora.strftime("%Y-%m-%d %H:%M"))
        
        return da.createVersion(nombre_version, "Protected", descripcion)

    @ToolboxLogger.log_method
    def obtenerConsultaInicial(self, campos, valores) :
        ToolboxLogger.debug("campos : '{}'".format(campos))
        ToolboxLogger.debug("valores: '{}'".format(valores))

        self.consultaInicial = QueryList()
        self.consultaInicial.addQuery(QueryItem(campos, valores))

    def registroEjecucion(self) :
        registro = {}
        registro["modulo"] = self.__module__
        registro["portal"] = self.portal
        registro["usuario"] = self.usuario
        registro["usuario_campo"] = self.usuario_campo
        registro["servicio_fuente"] = self.servicioFuente
        registro["version_fuente"] = self.versionFuente
        registro["servicio_destino"] = self.servicioDestino
        registro["version_destino"] = self.versionDestino
        registro["nodo"] = self.node
        registro["plataforma"] = self.plataforma
        registro["sistema"] = self.system
        registro["release"] = self.release
        registro["version"] = self.version
        registro["machine"] = self.machine
        registro["python_impl"] = self.python_impl
        registro["python_rev"] = self.python_rev
        registro["python_ver"] = self.python_ver
        registro["arcgis_ver"] = self.arcgis_ver
        registro["total_tablas"] = self.importarDatos.estadisticas.obtenerTotalTablas()
        registro["total_encuestas"] = self.importarDatos.estadisticas.obtenerNumeroEstadisticas()
        registro["total_creados"] = self.importarDatos.estadisticas.obtenerTotalEstadisticas("C")
        registro["total_actualizados"] = self.importarDatos.estadisticas.obtenerTotalEstadisticas("U")
        registro["total_no_actualizados"] = self.importarDatos.estadisticas.obtenerTotalEstadisticas("NU")
        registro["total_errores"] = self.importarDatos.estadisticas.obtenerTotalEstadisticas("E")
        registro["total_registros"] = self.importarDatos.estadisticas.obtenerTotalRegistros()
        registro["total_operaciones"] = self.importarDatos.estadisticas.obtenerTotalOperaciones()
        registro["inicio"] = "{}".format(self.importarDatos.estadisticas.timer.startTime)
        registro["fin"] = "{}".format(self.importarDatos.estadisticas.timer.endTime)
        registro["duracion_total"] = "{}".format(self.importarDatos.estadisticas.timer.timeSpan)

        if self.importarDatos.estadisticas.obtenerTotalRegistros() > 0:
            registro["duracion_por_encuesta"] = "{}".format(self.importarDatos.estadisticas.duracionPorEstadistica())
            registro["duracion_por_registro"] = "{}".format(self.importarDatos.estadisticas.duracionPorRegistro())
            registro["duracion_por_operacion"] = "{}".format(self.importarDatos.estadisticas.duracionPorOperacion())
            registro["minutos_por_encuesta"] = self.importarDatos.estadisticas.minutosPorEstadistica()
            registro["minutos_por_registro"] = self.importarDatos.estadisticas.minutosPorRegistro()
            registro["minutos_por_operacion"] = self.importarDatos.estadisticas.minutosPorOperacion()
            registro["encuestas_por_minuto"] = self.importarDatos.estadisticas.estadisticasPorMinuto()
            registro["registros_por_minuto"] = self.importarDatos.estadisticas.registrosPorMinuto()
            registro["operaciones_por_minuto"] = self.importarDatos.estadisticas.operacionesPorMinuto()
        
        return registro

    @ToolboxLogger.log_method
    def Ejecutar(self) :
        total_encuestas = 0
        total_registros = 0
        total_errores = 0
        patron_fecha_hora = "%Y-%m-%d %H:%M:%S"
        try: 
            ToolboxLogger.info("Iniciado: {}".format(datetime.now().strftime(patron_fecha_hora)))
            self.inicializarProceso()
            if self.importarDatos :
                mapeos = self.config.getConfigKey("mapeos")
                for mapeo in mapeos :
                    self.calcularConsultaInicial()
                    encuestas, registros, errores = self.importarDatos.procesarMapeo(mapeo, self.consultaInicial)
                    total_encuestas += encuestas
                    total_registros += registros
                    total_errores += errores 
            self.limpiarProceso()
        except Exception as e:
            ToolboxLogger.info("Error: {}".format(e))
        finally :
            ToolboxLogger.info("Terminado: {}".format(datetime.now().strftime(patron_fecha_hora)))
            
        return total_encuestas, total_registros, total_errores


from arcgis.gis import GIS
from arcgis import features
from arcgis import geometry
from arcgis.features import _version

from ImportarSurvey.DataAccess import DataAccess
from ImportarSurvey.Utiles import ToolboxLogger

DEFAULT_FILTER_STRING = "1=1"

class ArcGISPythonApiDataAccess(DataAccess) :

    @staticmethod
    def getGIS(url_portal = None, username = None, password = None) :
        try :
            if url_portal == None and username == None and password == None :
                gis = GIS("home")    
            elif username == None and password == None :
                gis = GIS(url_portal)
            else :
                gis = GIS(url_portal, username, password)
        except Exception as e:
            ToolboxLogger.info("ERROR: ---->'{}'".format(e))

        return gis

    def __init__(self, gis):
        super().__init__()
        self.__gis = gis
        self.__version = None
        self.__version_manager = None
        self.__feature_service = None 
        self.outSpatialReference = None
        self.hasZ = False
        self.returnZ = False

    def __find_table(self, table_name) : 
        if self.__feature_service == None:
            raise Exception("Capa o tabla '{}' no disponible.".format(table_name))
        elif self.__feature_service.layers == None:
            raise Exception("Capa o tabla '{}' no disponible.".format(table_name))

        table = [x for x in self.__feature_service.layers if x.properties["name"].lower() == table_name.lower()]

        if len(table) == 0 :
            table = [x for x in self.__feature_service.tables if x.properties["name"].lower() == table_name.lower()]

        if len(table) == 0:
            return None

        return table[0]

    def getTable(self, table_name) :
        return self.__find_table(table_name)

    def _search_da(self, table, fields, filter = None, return_geometry = True, spatial_reference = None) :
        features = []

        ToolboxLogger.debug("nombre : {}".format(table.properties.name))
        ToolboxLogger.debug("filtro : {}".format(filter))
        ToolboxLogger.debug("campos : {}".format(fields))

        if filter != "" :

            if filter == None  :
                filter = DEFAULT_FILTER_STRING

            if isinstance(fields, list) :
                outFields = str.join(",", fields)
            else :
                outFields = fields

            if not spatial_reference:
                spatial_reference = self.outSpatialReference

            if self.__version != None :
                gdb_version = self.__version.properties.versionName
                query = table.query(
                    where= filter, 
                    outFields = outFields, 
                    return_z = self.returnZ,
                    return_geometry = return_geometry, 
                    out_sr = spatial_reference,
                    gdb_version = gdb_version)
            else :
                query = table.query(
                    where = filter, 
                    outFields = outFields, 
                    return_z = self.returnZ,
                    return_geometry = return_geometry, 
                    out_sr = spatial_reference)

            features = []
            for feature in query.features :
                f = feature.attributes
                if return_geometry and table.properties.type != "Table":
                    f["geometry"] = feature.geometry
                    if feature.geometry and not "spatial_reference" in feature.geometry :
                        f["geometry"]["spatial_reference"] = query.spatial_reference
                features.append(f)

        ToolboxLogger.debug("registros : {}".format(len(features)))
        return features

    def getSpatialReference(self) :
        if self.__feature_service and self.__feature_service.layers:
            sr = self.__feature_service.spatialReference
            ToolboxLogger.info("Referencia Espacial --> {}".format(sr))
            return self.__feature_service.layers[0].properties.extent.spatialReference
        return None

    def __getHasZ(self) :
        if self.__feature_service and self.__feature_service.layers:
            featLayers = [x for x in self.__feature_service.layers if x.properties.type == "Feature Layer"]
            if featLayers:
                return featLayers[0].properties.hasZ
        return False

    def _getFeature(self, record, spatialReference = None) :
        _attributes = record.copy()

        f = features.Feature()

        if "geometry" in _attributes and _attributes["geometry"] != {}:
            geo_key = _attributes["geometry"]
            del _attributes["geometry"]

            if geo_key :
                if not "spatialReference" in geo_key and spatialReference:
                    geo_key["spatialReference"] = spatialReference
                if not "hasZ" in geo_key:
                    geo_key["hasZ"] = self.hasZ
                if self.hasZ:
                    if "rings" in geo_key:
                        for ring in geo_key["rings"]:
                            if len(ring[0]) < 3:
                                for xy in ring:
                                    xy.append(0)
                f.geometry = geometry.Geometry(geo_key)
               
        f.attributes = _attributes
        return f

    @ToolboxLogger.log_method
    def add(self, table, record, spatialReference = None) :
        if isinstance(table, str):
            table = self.__find_table(table)

        features = []
        if isinstance(record, list):
            for r in record:
                f = self._getFeature(r, spatialReference)
                features.append(f)
        else:
            f = self._getFeature(record, spatialReference)
            features.append(f)

        if self.__version != None :
            r = self.__version.edit(layer=table, adds=features)
        else :
            r = table.edit_features(adds=features)

        if isinstance(record, list):
            return r["addResults"] if r["addResults"] else None
        else: 
            return r["addResults"][0] if r["addResults"] else None

    @ToolboxLogger.log_method
    def update(self, table, record) :
        
        if isinstance(table, str):
            table = self.__find_table(table)
        
        features = []
        if isinstance(record, list):
            for r in record:
                f = self._getFeature(r)
                features.append(f)
        else:
            f = self._getFeature(record)
            features.append(f)

        if self.__version != None :
            r = self.__version.edit(layer=table, updates=features)
        else :
            r = table.edit_features(updates=features)

        if isinstance(record, list):
            return r["updateResults"] if r["updateResults"] else None
        else:
            return r["updateResults"][0] if r["updateResults"] else None 

    @ToolboxLogger.log_method
    def delete(self, table, record) :         
        if isinstance(table, str):
            table = self.__find_table(table)

        oid_field = table.properties.objectIdField
        ids = []

        if isinstance(record, list) :
            for r in record:
                ids.append(r[oid_field])
        else :
            ids.append(record[oid_field])     

        if ids != [] :
            if self.__version != None :
                r = self.__version.edit(layer=table, deletes=ids)
            else :
                r = table.edit_features(deletes=ids)

            if isinstance(record, list) :
                return r["deleteResults"] if r["deleteResults"] else None
            else :
                return r["deleteResults"][0] if r["deleteResults"] else None
        else :
            return []

    @ToolboxLogger.log_method
    def query(self, table, fields, filter = None, return_geometry = True, spatial_reference = None) :
        if isinstance(table, str) :
            table = self.__find_table(table)

        return self._search_da(table, fields, filter, return_geometry, spatial_reference)

    @ToolboxLogger.log_method
    def getServiceTables(self) :
        tables = [x for x in self.__feature_service.tables]

        return tables

    @ToolboxLogger.log_method
    def getServiceLayers(self) :
        tables = [x for x in self.__feature_service.layers]
        return tables

    @ToolboxLogger.log_method
    def queryRelatedRecords(self, table, record, relationshipId, fields = '*', expression = None) :
        if isinstance(table, str) :
            table = self.__find_table(table)

        oid_field = table.properties.objectIdField
        ids = []
        
        if isinstance(record, list) :
            for r in record:
                ids.append("{}".format(r[oid_field]))
        else :
            ids.append("{}".format(record[oid_field]))    

        if isinstance(fields, list):
            fields = ",".join(fields)

        ids = ",".join(ids)

        if self.__version != None :
            gdb_version = self.__version.properties.versionName
            r = table.query_related_records(ids, relationshipId, definition_expression=expression, out_fields=fields, gdb_version = gdb_version)
        else :
            r = table.query_related_records(ids, relationshipId, definition_expression=expression, out_fields=fields)

        result = []
        for r in r["relatedRecordGroups"]:
            item = {}
            item["objectid"] = r["objectId"]

            records = []
            for rr in r["relatedRecords"]: 
                f = rr["attributes"]
                if "geometry" in rr:
                    f["geometry"] = rr["geometry"]
                    
                records.append(f)
                    
            item["records"] = records
            result.append(item)

        return result

    @ToolboxLogger.log_method
    def setFeatureService(self, feature_service_name) :
        query = "title: '{}' AND type: 'Feature Service'".format(feature_service_name)
        search_results = self.__gis.content.search(query = query, max_items = 10)
        self.__feature_service = [x for x in search_results if x["title"] == feature_service_name][0] if search_results else None
        self.hasZ = self.__getHasZ()
        self.returnZ = self.hasZ

        return self.__feature_service

    @ToolboxLogger.log_method
    def copyFeatureService(self, new_service_name) :
        if(self.__feature_service != None) :
            return self.__feature_service.copy(title = new_service_name)
        else:
            return None

    @ToolboxLogger.log_method
    def setVersionManager(self) :

        if(self.__feature_service != None) :
            feature_service_url = self.__feature_service.url
            if feature_service_url != None :
                version_manager_service_url = feature_service_url.replace("FeatureServer", "VersionManagementServer")
                self.__version_manager = _version.VersionManager(version_manager_service_url, self.__gis, self.__feature_service)
        return self.__version_manager

    @ToolboxLogger.log_method
    def isVersionLocked(self, versionName) :
        try:
            if self.__version_manager :
                versions = [x for x in self.__version_manager.locks if x.properties.versionName.lower() == versionName.lower()]
                if(len(versions)) > 0:
                    version = versions[0]
                    ToolboxLogger.debug("version.properties.isBeingEdited: {}".format(version.properties.isBeingEdited))
                    ToolboxLogger.debug("version.properties.isBeingRead: {}".format(version.properties.isBeingRead))
                    ToolboxLogger.debug("version.properties.isLocked: {}".format(version.properties.isLocked))
                    ToolboxLogger.debug("version.mode: {}".format(version.mode))
                    if version.properties.isBeingEdited or version.properties.isBeingRead :
                        ToolboxLogger.info("{}".format(version.properties))
                        result = self.__version_manager.purge(version.properties.versionName)
                    return version.properties.isBeingEdited or version.properties.isBeingRead, result
                else :
                    return False, False
            else :
                raise Exception("VersionManager Nulo")
        except Exception as e:
            ToolboxLogger.info("ERROR: ---->'{}'".format(e))

    @ToolboxLogger.log_method
    def setVersion(self, versionName) :
        try:
            all_versions = [x for x in self.__version_manager.all if x.properties.versionName.lower() == versionName.lower()]
            if (len(all_versions) > 0) :
                self.__version = all_versions[0]
                self.__version.save_edits = True
            else:
                raise Exception("Versión No Existe!")

            return self.__version
        except Exception as e:
            ToolboxLogger.info("ERROR: ---->'{}'".format(e))

        return None 

    @ToolboxLogger.log_method
    def purgeVersion(self, versionName) :
        versionsToPurge = [x for x in self.__version_manager.all if x.properties.versionName.lower() == versionName.lower()]
        versionToPurge = versionsToPurge[0] if len(versionsToPurge) > 0 else None
        if versionToPurge:
            ToolboxLogger.debug("version.properties.isBeingEdited: {}".format(versionToPurge.properties.isBeingEdited))
            ToolboxLogger.debug("version.properties.isBeingRead: {}".format(versionToPurge.properties.isBeingRead))
            ToolboxLogger.debug("version.properties.isLocked: {}".format(versionToPurge.properties.isLocked))
            ToolboxLogger.debug("version.mode: {}".format(versionToPurge.mode))
            
            result = self.__version_manager.purge(versionToPurge.properties.versionName) if versionToPurge.mode == None and versionToPurge.properties.isLocked else True
        return result
        
    @ToolboxLogger.log_method
    def createVersion (self, versionName, permission="public", description ="") :
        result = self.__version_manager.create(versionName, permission, description)

        if result : 
            new_versions = [x for x in self.__version_manager.all if x.properties.versionName.__contains__(versionName)]
            return new_versions[0].properties.versionName if new_versions != None else None 
        else:
            return None

    @ToolboxLogger.log_method
    def deleteVersions(self, propietario, encuesta) :
        versions = [x for x in self.__version_manager.all if x.properties.versionName.__contains__(propietario + ".") and x.properties.versionName.__contains__(encuesta)]

        for version in versions:
            ToolboxLogger.debug("eliminando '{}'".format(version.properties.versionName))
            if version.properties.isLocked == True :
                self.__version_manager.purge(version.properties.versionName, propietario)
            version.delete()

    @ToolboxLogger.log_method
    def setMode(self, modo) :
        if self.__version :
            self.__version.mode = modo

    @ToolboxLogger.log_method
    def startEditing(self):
        if self.__version :
            try:
                result = False
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                result = self.__version.start_editing()
            except Exception as ex:
                ToolboxLogger.info(ex)
            finally :
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                return result

    @ToolboxLogger.log_method
    def stopEditing(self, save_edits):
        if self.__version :
            try:
                result = False
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                result = self.__version.stop_editing(save_edits)
            except Exception as ex:
                ToolboxLogger.info(ex)
            finally :
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                return result

    @ToolboxLogger.log_method
    def startReading(self):
        if self.__version :
            try:
                result = False
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                result = self.__version.start_reading()
            except Exception as ex:
                ToolboxLogger.info(ex)
            finally :
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                return result 

    @ToolboxLogger.log_method
    def stopReading(self):
        if self.__version :
            try:
                result = False
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                result = self.__version.stop_reading()
            except Exception as ex:
                ToolboxLogger.info(ex)
            finally :
                ToolboxLogger.debug("version.mode: {}".format(self.__version.mode))
                return result

    @ToolboxLogger.log_method
    def describeRelation(self, table_name, idRelacion) :
        relacion = None
        if idRelacion :
            table = self.__find_table(table_name)
            relaciones = [x for x in table.properties.relationships if x.id == idRelacion] 
            if len(relaciones) > 0 :
                for relacion in relaciones :
                    ToolboxLogger.debug("relacion id: {} - nombre: '{}'".format(relacion.id, relacion.name))
        return relacion

    def getElementsInFolder(self, folder_name) :   
        list_elements = self.__gis.content.items(folder=folder_name, max_items=10)
        return list_elements
    
    @ToolboxLogger.log_method
    def searchElement(self, element_name, element_type) :
        query = "title: '{}' AND type: '{}'".format(element_name, element_type)
        search_results = self.__gis.content.search(query=query, max_items=10)
        
        return search_results

    @ToolboxLogger.log_method
    def copyElement(self, element, new_element_name) :
        if(element != None) :
            return element.copy(title=new_element_name)
        else:
            return None
    



# -*- coding: utf-8 -*-

from psutil import WINDOWS

if WINDOWS: 
  import arcpy

import logging
import sys
import os
import json

from datetime import datetime
from os import path
from logging import handlers

#### CONSTANTES ######
CONFIG_PATH = ""
LOG_PATH = ""
LOG_FILE = ""

ARCGIS_HANDLER = 1
STREAM_HANDLER = 2
FILE_HANDLER = 4

class ToolboxLogger :

  _logger = None

  @classmethod
  def log_method(cls, func) :
    cls.indent = ""
    cls.indentSize = 2

    def inner(*args, **kwargs) :
      cls._logger.debug("{}-->{}".format(cls.indent, func.__qualname__))
      cls.indent += " ".ljust(cls.indentSize)
      try :
        result = func(*args, **kwargs)
      except Exception as e:
        raise e
      finally :
        cls.indent = cls.indent[:-2]
        cls._logger.debug("{}<--{}".format(cls.indent, func.__qualname__))

      return result
    
    return inner

  @classmethod
  def initLogger(cls, source = "ToolboxLogger", log_path = LOG_PATH, log_file = LOG_FILE, handler_type = ARCGIS_HANDLER) :
    loggerFactory = LoggerFactory()
    cls._logger = loggerFactory.getLogger(source, log_path, log_file, handler_type = handler_type)
    cls.indent = ""
    cls.indentSize = 2

  @classmethod
  def setInfoLevel(cls) :
    cls._logger.logLevel = logging.INFO
    cls._logger.setLevel(logging.INFO)
    cls.info("Level: INFO")
    
    for h in cls._logger.handlers :
      h.setLevel(cls._logger.logLevel)

  @classmethod
  def setDebugLevel(cls) :
    cls._logger.logLevel = logging.DEBUG
    cls._logger.setLevel(logging.DEBUG)
    cls.info("Level: DEBUG")

    for h in cls._logger.handlers :
      h.setLevel(cls._logger.logLevel)

  @classmethod
  def debug(cls, message) :
    cls._logger.debug("%s", "{}{}".format(cls.indent, message))

  @classmethod
  def info(cls, message) :
    cls._logger.info("%s", "{}{}".format(cls.indent, message))

  @classmethod
  def error(cls, message) :
    cls._logger.error("%s", "{}{}".format(cls.indent, message))

  @classmethod
  def warning(cls, message) :
    cls._logger.warging("%s", "{}{}".format(cls.indent, message))


if WINDOWS:
  class ArcGisLogHandler(logging.Handler): 
      def __init__(self): 
          logging.Handler.__init__(self)
          
      def emit(self, record): 
          if record.levelno >= logging.ERROR: 
              log_method = arcpy.AddError 
          elif record.levelno >= logging.WARNING: 
              log_method = arcpy.AddWarning 
          else: 
              log_method = arcpy.AddMessage 
          log_method(self.format(record))

class LoggerFactory : 
  #self.pattern = '%(asctime)s - %(message)s'
  pattern = '%(message)s'

  def clearLogger(self, name) :
    logger = logging.getLogger(name)
    logger.handlers.clear()

  def getLogger(self, name, pathName, fileName, handler_type = ARCGIS_HANDLER) :
    self.logLevel = logging.INFO

    logger = logging.getLogger(name)
    logger.setLevel(self.logLevel)

    if len(logger.handlers) > 0:  

      if handler_type & FILE_HANDLER == FILE_HANDLER:
        if not any(map(lambda h : h.__class__.__name__ == "RotatingFileHandler", logger.handlers)) :
          self.addRotatingFileHandler(logger, pathName, fileName)
        
      if WINDOWS and (handler_type & ARCGIS_HANDLER == ARCGIS_HANDLER):
        if not any(map(lambda h : h.__class__.__name__ == "ArcGisLogHandler", logger.handlers)) :
          self.addArcGisLogHandler(logger)

      if handler_type & STREAM_HANDLER == STREAM_HANDLER:
        if not any(map(lambda h : h.__class__.__name__ == "StreamHandler", logger.handlers)) :
            self.addStreamHandler(logger)
    else:
      if handler_type & FILE_HANDLER == FILE_HANDLER:
        self.addRotatingFileHandler(logger, pathName, fileName)
      
      if WINDOWS and (handler_type & ARCGIS_HANDLER == ARCGIS_HANDLER):
        self.addArcGisLogHandler(logger)

      if handler_type & STREAM_HANDLER == STREAM_HANDLER:
        self.addStreamHandler(logger)

    return logger

  def addRotatingFileHandler(self, logger, pathName, fileName) :
    fullFileName = os.path.join(pathName, "{}{}.log".format(fileName, datetime.now().strftime("%Y%m%d%H%M")))
    chs = handlers.RotatingFileHandler(filename = fullFileName, backupCount = 52, maxBytes = 1000000, encoding='utf-8')
    chs.setFormatter(logging.Formatter(self.pattern))
    chs.setLevel(self.logLevel)
    logger.addHandler(chs)
    logger.debug("Agregando RotatingFileHandler")
  
  def addStreamHandler(self, logger) :
      ch = logging.StreamHandler(sys.stdout)                             
      ch.setFormatter(logging.Formatter(self.pattern))
      ch.setLevel(self.logLevel)
      logger.addHandler(ch)
      logger.info("Agregando StreamHandler")

  def addArcGisLogHandler(self, logger) :
    if WINDOWS: 
      ch = ArcGisLogHandler()
      ch.setFormatter(logging.Formatter(self.pattern))
      ch.setLevel(self.logLevel)
      logger.addHandler(ch)
      logger.info("Agregando ArcGisLogHandler")
    else :
      pass

class Configuracion:

    def __init__(self, configPath):        
        try:
            _path = os.path.dirname(os.path.abspath(__file__))
            if(configPath == None) :
              self.path = CONFIG_PATH
            else :
              self.path = configPath

            _file = open(self.path, encoding='utf-8')
            config = _file.read()
            
            self.configs = json.loads(config)

        except Exception as d:
            self.WriteMessage("Archivo de configuracion no es válido, {}".format(str(d)))
            raise d

    def getConfigKey(self, key):
        try:
          value = self.configs[key]
          return value
        except Exception as d:            
          self.WriteMessage("La llave ''{}'' no existe.\n{}".format(key, str(d)))
          pass
            
    def WriteMessage(self, msg) :
        ToolboxLogger.info(msg)    

class JsonFile :

  @staticmethod
  def readFile(filename):
    records = []

    if os.path.exists(filename): 
      try :
        with open(filename, encoding='utf-8') as f:
          text = f.read()
          records  = json.loads(text)
      except :
        records = []
    return records

  @staticmethod
  def writeFile(filename, records) :
    with open(filename, 'w') as f:
      text = json.dumps(records, indent = 4)
      f.write(text)

class TimeUtil :

  startTime = None
  endTime = None 
  timeSpan = None

  def __init__(self) :
    self.initTimer()

  def initTimer(self) :
    self.startTime  = datetime.now()

  def stopTimer(self) :
    self.endTime = datetime.now()
    self.timeSpan = self.endTime - self.startTime
    return self.timeSpan

  @staticmethod
  def nowCode() :
    return datetime.now().strftime("%Y%m%d%H%M%S")



import arcpy
import arcgis

from arcgis.gis import GIS
from arcgis import features
from arcgis import geometry
from datetime import datetime

from ImportarSurvey.DataAccess import DataAccess
from ImportarSurvey.Utiles import ToolboxLogger

DEFAULT_FILTER_STRING = "1=1"

class ArcGISWFMApiDataAccess(DataAccess) :
    def __init__(self, pathConnectionFile = None) :
        DataAccess.__init__(self)

        if pathConnectionFile == None :
            self.__conn =  arcpy.wmx.Connect()
        else :
            self.__conn =  arcpy.wmx.Connect(pathConnectionFile)

    def _getJobsByUser(self, userName):
        jobsUser = self.__conn.queryJobs("JOB_ID,JOB_NAME,ASSIGNED_TO,JOB_TYPE_ID,ASSIGNED_TYPE,STATUS,STAGE, START_DATE, DUE_DATE, STARTED_DATE, END_DATE, PERC_COMPLETE","JTX_JOBS","ID,Job Name,Assigned To, Job Type, Assigned Type, Status, Stage, Start Date, Due Date, Started Date, End Date, Perc Complete ", "ASSIGNED_TO = '" + userName + "'", "JOB_NAME")
        return jobsUser
	
    def _getAssignmentsUser(self, userName):
        assignmentsUser = self.__conn.queryJobs("JOB_ID,ID_SOLICITUD, ID_TRAMITE, TIPO_TRAMITE","WMX_EC_GESTIONEDICION","ID_Job,ID_Solicitud,ID_Tramite,Tipo_Tramite", "EDITOR_ASIGNADO = '" + userName + "'", "JOB_ID")
        return assignmentsUser

    def _getRequestById(self, requestID):
        
        requestData = self.__conn.queryJobs("JOB_ID,ID_SOLICITUD, ID_TRAMITE, TIPO_TRAMITE, CEDULA_CATASTRAL, VERSION","WMX_EC_GESTIONEDICION","ID_Job,ID_Solicitud,ID_Tramite,Tipo_Tramite,Cedula_Catastral,Version", "ID_Solicitud = '" + requestID + "'", "JOB_ID")
        return requestData  

    def __find_table(self, table_name) : 
        if self.__feature_service == None:
            raise Exception("Capa o tabla '{}' no disponible.".format(table_name))
        elif self.__feature_service.layers == None:
            raise Exception("Capa o tabla '{}' no disponible.".format(table_name))

        table = [x for x in self.__feature_service.layers if x.properties["name"].lower() == table_name.lower()]

        if len(table) == 0 :
            table = [x for x in self.__feature_service.tables if x.properties["name"].lower() == table_name.lower()]

        if len(table) == 0:
            return None

        return table[0]

    def getTable(self, table_name) :
        return self.__find_table(table_name)

    def _search_da(self, table, fields, filter = None, return_geometry = True) :
        features = []

        ToolboxLogger.debug("nombre : {}".format(table.properties.name))
        ToolboxLogger.debug("filtro : {}".format(filter))
        ToolboxLogger.debug("campos : {}".format(fields))

        if filter != "" :

            if filter == None  :
                filter = DEFAULT_FILTER_STRING

            if isinstance(fields, list) :
                outFields = str.join(",", fields)
            else :
                outFields = fields

            if self.__version != None :
                gdb_version = self.__version.properties.versionName
                query = table.query(where= filter, outFields = outFields, return_geometry = return_geometry, gdb_version = gdb_version)
            else :
                query = table.query(where = filter, outFields = outFields, return_geometry = return_geometry)

            features = []
            for feature in query.features :
                f = feature.attributes
                if feature.geometry:
                    f["geometry"] = feature.geometry
                features.append(f)

        ToolboxLogger.debug("len(query)) : {}".format(len(features)))
        return features

    def _getFeature(self, record) :
        record_copy = record.copy()
        if "geometry" in record_copy:
            geo = record_copy["geometry"]
            del record_copy["geometry"]

            return features.Feature(attributes=record_copy, geometry=geometry.Geometry(geo))
        else :
            return features.Feature(attributes=record_copy)

        itemProject = self.__gis.content.get(projectID)
        project = arcgis.apps.workforce.Project(itemProject)
        return project


# -*- coding: utf-8 -*-

DEFAULT_OPERATOR = "="

class QueryFilter:
    def __init__(self, query_filter) :
        self._query_filter = query_filter

    def getWhereClause(self) :
        def __process_clausule(item) : 
            fields = []
            values = []
            operators = []
            connectors = []

            field = item["query"]["field"]
            value = item["query"]["value"] if "value" in item["query"] else None
            operator = item["query"]["operator"] if "operator" in item["query"] else None
            connector = item["connector"] if "connector" in item else None
            queries = item["queries"] if "queries" in item else []
            for query in queries:
               f, v, o, c, cf, cv, co, cc = __process_clausule(query)
               fields.append(f)
               values.append(v)
               operators.append(o)
               connectors.append(c)

               if cf != [] :
                   fields.append(cf)
               if cv != [] :
                   values.append(cv)
               if co != [] :
                   operators.append(co)
               if cc != [] :
                   connectors.append(cc)

            return field, value, operator, connector, fields, values, operators, connectors

        fields = []
        values = []
        operators = []
        connectors = []
        for item in self._query_filter["queries"] :
            f, v, o, c, cf, cv, co, cc = __process_clausule(item)
            fields.append(f)
            values.append(v)
            operators.append(o)
            connectors.append(c)
            if cf != [] :
                fields.append(cf)
            if cv != [] :
                values.append(cv)
            if co != [] :
                operators.append(co)
            if cc != [] :
                connectors.append(cc)

        return DataAccess.getWhereClause(fields, values, operators, connectors)

class QueryItem :
    def __init__(self, field, values, operator = None): 
        self._field = field
        self._values = values
        self._operator = operator

    def getWhereClause(self) :
        return DataAccess.getWhereClause(self._field, self._values, self._operator)

class QueryList :
    def __init__(self) :
        self._queries = []

    def addQuery(self, queryItem, connector = None) :
        if isinstance(queryItem._field, list) and isinstance(queryItem._values, list):
            index = 0
            for f in queryItem._field :
                v = queryItem._values[index]
                o = queryItem._operator[index] if queryItem._operator else None
                self._queries.append({"item": QueryItem(f, v, o), "connector": connector})
                index += 1
        else: 
            self._queries.append({"item": queryItem, "connector": connector})

    def getWhereClause(self):
        if len(self._queries) > 0 :
            fields = list(map(lambda q : q["item"]._field, self._queries))
            values = list(map(lambda q : q["item"]._values, self._queries))
            operators = list(map(lambda q : q["item"]._operator if q["item"]._operator else None, self._queries))
            connectors = list(map(lambda  q: q["connector"] if q["connector"] else "", self._queries))

            return DataAccess.getWhereClause(fields, values, operators, connectors)
        else :
            return None

class DataAccess :

    @classmethod
    def _getPartialWhereClause(cls, field, value, operator = None, connectors = None):
        if isinstance(field, list) and isinstance(value, list) :
            if not operator :
                operator = []
                for f in field:
                    operator.append(None)
            if not connectors :
                connectors = []
                for f in field:
                    connectors.append(None)

            where = cls._getWhereClauseMulti(field, value, operator, connectors)

        elif isinstance(value, list):
            value = list(map(lambda v: "'{}'".format(v) if isinstance(v, str) else "{}".format(v), value))
            where = "{} in ({})".format(field, ", ".join(value))
        elif isinstance(value, str): 
            if not operator :
                operator = DEFAULT_OPERATOR
            if "NULL" in value.upper():
                where = "{} {} {}".format(field, operator, value) if value else "{} {} ''".format(field, operator)
            else:
                where = "{} {} '{}'".format(field, operator, value) if value else "{} {} ''".format(field, operator)
        else:
            if not operator :
                operator = DEFAULT_OPERATOR
            where = "{} {} {}".format(field, operator, value) if value else "{} {} ''".format(field, operator)
        return where

    @classmethod
    def getWhereClause(cls, field, value, operator = None, connectors = None):
        if isinstance(field, str):
            return cls._getPartialWhereClause(field, value, operator)

        if isinstance(field, list) and isinstance(value, list) :
            return cls._getWhereClauseMulti(field, value, operator, connectors)

        return ""
    
    @classmethod
    def _getWhereClauseMulti(cls, fields, values, operators = None, connectors = None) :
        if len(fields) <= len(values):
            if not operators:
                operators = []

            if len(operators) < len(fields) :
                count = len(fields) - len(operators)
                index = 0
                while index < count :
                    index += 1
                    operators.append(None)

            if not connectors:
                connectors = []

            if len(connectors) < len(fields) :
                count = len(fields) - len(connectors)
                index = 0
                while index < count :
                    index += 1
                    connectors.append(None)
                
            if operators and connectors:
                clauses = list(map(cls._getPartialWhereClause, fields, values, operators, connectors))
            else :
                clauses = list(map(cls._getPartialWhereClause, fields, values))
            
            emptyValues = [c for c in clauses if c == ""]
            if len(emptyValues) < 1 :
                if len(clauses) == 1:
                    return clauses[0]
                elif not connectors:
                    where = " AND ".join(clauses)
                    return "({})".format(where.strip())
                else :
                    if len(connectors) == len(clauses):
                        connectors.pop()
                    where = ""
                    i = 0
                    for clause in clauses:
                        if i >= len(connectors) :
                            connector = ""
                        elif not connectors[i] :
                            connector = "AND"
                        else :
                            connector = connectors[i]
                        where += "{} {} ".format(clause, connector.strip())
                        i += 1
                    return "({})".format(where.strip())
                        
        return ""

    @classmethod
    def enumElements(self, elements):
        element_list = []
        if elements:
            if isinstance(elements, list) :
                for element in elements:
                    if not isinstance(element, list) :
                        element_list.append(element)
                    else :
                        sub_list = self.enumElements(element)
                        element_list = element_list + sub_list
            else :
                element_list.append(elements)
        return element_list

    def __init__(self) :
        pass

    def query(self, table, fields, filter = None) :
        pass

    def update(self, tbl_destino, campo_busqueda, campos, registro_origen, valor_relacion, tipo_operacion) :
        pass
            




# -*- coding: utf-8 -*-
from ImportarSurvey.Utiles import TimeUtil, ToolboxLogger

class Estadisticas:

    def __init__(self) :
        self.estadisticas = []
        self.tablasEstadisticas = []
        self.encuestas = []
        self.valores = {"E": 3, "C": 2, "U": 1, "NU" : 0}
        self.timer = TimeUtil()
    
    def agregarEncuesta(self, idEncuesta):
        encuesta = {}
        encuesta["idEncuesta"] = idEncuesta
        self.encuestas.append(encuesta)

        return encuesta

    def actualizarEncuesta(self, idEncuesta, estados) :
        encuestas = [x for x in self.encuestas if x["idEncuesta"] == idEncuesta]
        encuesta = encuestas[0] if len(encuestas) > 0 else None
        if encuesta:
            for llave in estados:
                encuesta[llave] = estados[llave]
            ToolboxLogger.debug("encuesta {}".format(encuesta))

    def totalEstadoEncuesta(self, llave_estado, valor_estado) :
        cuenta = 0
        for encuesta in self.encuestas :
            if llave_estado in encuesta :
                cuenta = cuenta + 1 if encuesta[llave_estado] == valor_estado else cuenta
        return cuenta

    def agregarEstadistica(self) :
        estadistica = {}
        self.estadisticas.append(estadistica)

        return estadistica

    def _obtenerEstadisticaActual(self):
        indice = len(self.estadisticas) - 1
        if indice >= 0:
            return self.estadisticas[indice]
        else :
            return None

    def _obtenerValor(self, tipo) :
        return self.valores[tipo] if tipo in self.valores else -1

    def actualizarEstadistica(self, nombreTabla, tipo, guid = "{}"):
        valor = self._obtenerValor(tipo)
        estadistica = self._obtenerEstadisticaActual()

        if estadistica != None :
            if not nombreTabla in self.tablasEstadisticas :
                self.tablasEstadisticas.append(nombreTabla)

            if not nombreTabla in estadistica:
                estadistica[nombreTabla] = {"guids": {}, "total" : 0}
                
            if valor > -1 :
                if not guid in estadistica[nombreTabla]["guids"] or valor > estadistica[nombreTabla]["guids"][guid]:
                    estadistica[nombreTabla]["guids"][guid] = valor 

                ToolboxLogger.debug("tabla: '{}' tipo: '{}' guid: {}".format(nombreTabla, tipo, guid))
                estadistica[nombreTabla]["total"] += 1

    def obtenerEstadisticas(self, indice, tipo) :
        cuenta = 0
        valor = self._obtenerValor(tipo)

        estadistica = self.estadisticas[indice]
        for tabla in estadistica:
            for guid in estadistica[tabla]["guids"] :
                if estadistica[tabla]["guids"][guid]  == valor :
                    cuenta += 1

        return cuenta

    def obtenerRegistros(self, indice) :
        cuenta = 0

        estadistica = self.estadisticas[indice]
        for tabla in estadistica:
            for guid in estadistica[tabla]["guids"] :
                cuenta += 1

        return cuenta

    def obtenerOperaciones(self, indice) :
        cuenta = 0

        estadistica = self.estadisticas[indice]
        for tabla in estadistica:
            cuenta = cuenta + estadistica[tabla]["total"] 

        return cuenta

    def obtenerEstadisticasActual(self, tipo) :
        cuenta = 0
        valor = self._obtenerValor(tipo)

        estadistica = self._obtenerEstadisticaActual()
        for tabla in estadistica:
            for guid in estadistica[tabla]["guids"] :
                if estadistica[tabla]["guids"][guid]  == valor :
                    cuenta += 1

        return cuenta

    def obtenerRegistrosActual(self) :
        cuenta = 0
        estadistica = self._obtenerEstadisticaActual()
        for tabla in estadistica:
            for guid in estadistica[tabla]["guids"] :
                cuenta += 1

        return cuenta

    def obtenerOperacionesActual(self) :
        cuenta = 0

        estadistica = self._obtenerEstadisticaActual()
        for tabla in estadistica:
            cuenta = cuenta + estadistica[tabla]["total"] 

        return cuenta
    
    def obtenerTotalEstadisticas(self, tipo) :
        cuenta = 0
        valor = self._obtenerValor(tipo)

        for estadistica in self.estadisticas:
            for tabla in estadistica:
                for guid in estadistica[tabla]["guids"] :
                    if estadistica[tabla]["guids"][guid]  == valor :
                        cuenta += 1

        return cuenta

    def obtenerTotalRegistros(self) :
        cuenta = 0
        for estadistica in self.estadisticas:
            for tabla in estadistica:
                for guid in estadistica[tabla]["guids"] :
                    cuenta += 1

        return cuenta

    def obtenerTotalOperaciones(self) :
        cuenta = 0
        for estadistica in self.estadisticas:
            for tabla in estadistica:
                cuenta = cuenta + estadistica[tabla]["total"] 

        return cuenta

    def obtenerTotalEstadisticasTabla(self, tabla, tipo) :
        cuenta = 0
        valor = self._obtenerValor(tipo)

        for estadistica in self.estadisticas:
            if tabla in estadistica :
                for guid in estadistica[tabla]["guids"] :
                    if estadistica[tabla]["guids"][guid]  == valor :
                        cuenta += 1
        return cuenta

    def obtenerTotalRegistrosTabla(self, tabla) :
        cuenta = 0
        for estadistica in self.estadisticas:
            if tabla in estadistica :
                for guid in estadistica[tabla]["guids"] :
                    cuenta += 1

        return cuenta

    def obtenerTotalOperacionesTabla(self, tabla) :
        cuenta = 0
        for estadistica in self.estadisticas:
            if tabla in estadistica :
                cuenta = cuenta + estadistica[tabla]["total"] 

        return cuenta
    
    def obtenerTotalTablas(self) :
        return len(self.tablasEstadisticas)

    def obtenerNumeroEstadisticas(self) :
        return len(self.estadisticas)

    def duracionPorEstadistica(self) :
        return self.timer.timeSpan / self.obtenerNumeroEstadisticas()
    
    def duracionPorRegistro(self) :
        return self.timer.timeSpan / self.obtenerTotalRegistros()

    def duracionPorOperacion(self) :
        return self.timer.timeSpan / self.obtenerTotalOperaciones()

    def minutosPorEstadistica(self) :
        return 1 / self.estadisticasPorMinuto() if self.estadisticasPorMinuto() != 0 else 0

    def minutosPorRegistro(self) :
        return 1 / self.registrosPorMinuto() if self.registrosPorMinuto() != 0 else 0
    
    def minutosPorOperacion(self) :
        return 1 / self.operacionesPorMinuto() if self.operacionesPorMinuto() != 0 else 0

    def estadisticasPorMinuto(self) :
        return self.obtenerNumeroEstadisticas() / self.timer.timeSpan.total_seconds() * 60 if self.timer.timeSpan.total_seconds() != 0 else 0

    def registrosPorMinuto(self) :
        return self.obtenerTotalRegistros() / self.timer.timeSpan.total_seconds() * 60 if self.timer.timeSpan.total_seconds() != 0 else 0

    def operacionesPorMinuto(self) :
        return self.obtenerTotalOperaciones() / self.timer.timeSpan.total_seconds() * 60 if self.timer.timeSpan.total_seconds() != 0 else 0


# -*- coding: utf-8 -*-

from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess
from ImportarSurvey.ImportarDatosPrecarga import ImportarDatosPrecarga
from ImportarSurvey.ProcesarEncuestas import ProcesarEncuestas

class PrecargarEncuestas(ProcesarEncuestas):
    
    def __init__(self, 
        rutaConfiguracion, 
        portal = None, 
        usuario = None, 
        clave = None, 
        servicioFuente = None, 
        servicioDestino = None, 
        versionFuente = None, 
        versionDestino = None, 
        usuarioCampo = None, 
        idsPrecarga = None):

        super().__init__(
            rutaConfiguracion, 
            portal, 
            usuario, 
            clave, 
            servicioFuente,  
            servicioDestino, 
            versionFuente = versionFuente,
            versionDestino = versionDestino, 
            usuarioCampo = usuarioCampo)
        self.idsPrecarga = idsPrecarga

    @ToolboxLogger.log_method
    def calcularConsultaInicial(self) :
        campos = "NUMERO_PREDIAL"
        valores = []
        
        if isinstance(self.idsPrecarga, str) :
            separadores = ';.,'
            for separador in separadores:
                self.idsPrecarga = self.idsPrecarga.replace(separador, ' ')
            valores = self.idsPrecarga.split(' ')
        elif isinstance(self.idsPrecarga, list) :
            valores = self.idsPrecarga
        
        self.obtenerConsultaInicial(campos, valores)

        return

    @ToolboxLogger.log_method
    def inicializarProceso(self) :
        super().inicializarProceso()
        
        try :
            ToolboxLogger.info("Portal: {}".format(self.portal))
            ToolboxLogger.info("Servicio Origen: {}".format(self.servicioFuente))
            ToolboxLogger.info("Servicio Destino: {}".format(self.servicioDestino))
            ToolboxLogger.info("Usuario: {}".format(self.usuario))
            ToolboxLogger.info("Nombre Usuario: '{}'".format(self.nombreCompleto))
            ToolboxLogger.info("Usuario Campo: {}".format(self.usuario_campo))
            ToolboxLogger.info("Versión Fuente: {}".format(self.versionFuente))
            
            self.fuente_da = ArcGISPythonApiDataAccess(self.gis)
            self.fuente_da.setFeatureService(self.servicioFuente)
            self.fuente_da.setVersionManager()

            if not self.versionFuente:
                self.versionFuente = self.crearVersion(self.fuente_da, self.servicioFuente)
                ToolboxLogger.info("Se creó la version: {}".format(self.versionFuente))
            else :
                ToolboxLogger.info("Versión Fuente: {}".format(self.versionFuente))

            if self.fuente_da.setVersion(self.versionFuente):
                self.destino_da = ArcGISPythonApiDataAccess(self.gis)
                self.destino_da.setFeatureService(self.servicioDestino)
                self.fuente_da.outSpatialReference = self.destino_da.getSpatialReference()
                self.fuente_da.returnZ = self.destino_da.hasZ
                
                ToolboxLogger.debug("Referencia Espacial Destino : '{}'".format(self.fuente_da.outSpatialReference))
                ToolboxLogger.debug("Tiene Z Destino : '{}'".format( self.fuente_da.returnZ))

                self.importarDatos = ImportarDatosPrecarga(
                    self.fuente_da, 
                    self.destino_da, 
                    self.servicioFuente, 
                    self.servicioDestino, 
                    self.usuario_campo)
            else:
                ToolboxLogger.info("Error: Versión Fuente No encontrada!")
        except Exception as e:
            ToolboxLogger.info("Error: {}".format(e))

        return


                


import arcpy
import os
import shutil

PYT_PATH = "../ImportarSurvey.pyt"

folder_path = os.path.dirname(os.path.realpath(__file__))
pyt_path = os.path.normpath(os.path.join(folder_path, PYT_PATH))
pyt_folder_path, pyt_filename_ext = os.path.split(pyt_path)
pyt_filename, ext = os.path.splitext(pyt_filename_ext)

source_folder_path = os.path.join(pyt_folder_path, 'esri')
target_folder_path = os.path.join(folder_path, 'esri')

try:
    arcpy.gp.createtoolboxsupportfiles(pyt_path)

    if os.path.isdir(target_folder_path):
        print('{} ya existe'.format(target_folder_path))

        shutil.rmtree(target_folder_path)   
        print("Eliminado: '{}'".format(target_folder_path))

    shutil.move(source_folder_path, folder_path)
    print("Se movió '{}' a '{}'".format(source_folder_path, folder_path))

    toolboxes_path = os.path.join(target_folder_path, 'toolboxes')
    os.mkdir(toolboxes_path)
    print("Se creó '{}'".format(toolboxes_path))

    pyt_files = [x for x in os.listdir(pyt_folder_path) if x.__contains__(pyt_filename) and os.path.isfile(os.path.join(pyt_folder_path, x))]

    for f in pyt_files:
        shutil.copy(os.path.join(pyt_folder_path, f), toolboxes_path)
        print("Se copió '{}' a '{}'".format(f, toolboxes_path))

except Exception as e:
    print("Error: {}".format(e))


# -*- coding: utf-8 -*-

from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.ImportarDatos import ImportarDatos

class ImportarDatosPrecarga(ImportarDatos) :

    def __init__(self, 
        fuente_da, 
        destino_da, 
        claseFuentePrincipal = '', 
        claseDestinoPrincipal = '', 
        usuarioCampo = '') :

        super().__init__(
            fuente_da,
            destino_da,
            claseFuentePrincipal = claseFuentePrincipal,
            claseDestinoPrincipal = claseDestinoPrincipal,
            usuarioCampo = usuarioCampo)

    @ToolboxLogger.log_method
    def actualizarRegistroDestino(self, tabla, registro, llave, valor = None) :
        try :
            if tabla == self.claseDestinoPrincipal and self.usuarioCampo != '' :
                llave = "usuario_portal" 
                valor = self.usuarioCampo

                if valor and llave in registro :
                    registro[llave] = valor
                    resultado = self.destino_da.update(tabla, registro)
                    ToolboxLogger.debug("Se actualizó el usuario de la encuesta : '{}'".format(valor))
                else :
                    ToolboxLogger.debug("No se actualizó.")

        except Exception as e:
            ToolboxLogger.info("Error: {}".format(e))



# -*- coding: utf-8 -*-
r""""""
__all__ = ['HerramientaInyectarEncuestas', 'HerramientaPrecargarEncuestas']
__alias__ = 'ImportarSurvey'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptooldoc('HerramientaInyectarEncuestas_ImportarSurvey', None)
def HerramientaInyectarEncuestas(portal=None, usuario=None, clave=None, servicioFuente=None, servicioDestino=None, versionDestino=None, usuarioCampo=None, informeDetallado=None):
    """HerramientaInyectarEncuestas_ImportarSurvey(portal, usuario, clave, servicioFuente, servicioDestino, {versionDestino}, usuarioCampo, {informeDetallado})

     INPUTS:
      portal (Cadena):
          URL del Portal: 
      usuario (Cadena):
          Usuario: 
      clave (Cadena de caracteres oculta):
          Clave: 
      servicioFuente (Cadena):
          Servicio Fuente (Encuesta Survey123):
      servicioDestino (Cadena):
          Servicio Destino (SubModelo Levantamiento Catastral):
      versionDestino {Cadena}:
          Versión Destino: 
      usuarioCampo (Cadena):
          Usuario Reconocedor: 
      informeDetallado {Booleano}:
          Informe Detallado:"""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.HerramientaInyectarEncuestas_ImportarSurvey(*gp_fixargs((portal, usuario, clave, servicioFuente, servicioDestino, versionDestino, usuarioCampo, informeDetallado), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('HerramientaPrecargarEncuestas_ImportarSurvey', None)
def HerramientaPrecargarEncuestas(portal=None, usuario=None, clave=None, servicioFuente=None, versionFuente=None, servicioDestino=None, usuarioCampo=None, idsPrecarga=None, informeDetallado=None):
    """HerramientaPrecargarEncuestas_ImportarSurvey(portal, usuario, clave, servicioFuente, {versionFuente}, servicioDestino, usuarioCampo, idsPrecarga;idsPrecarga..., {informeDetallado})

     INPUTS:
      portal (Cadena):
          URL del Portal: 
      usuario (Cadena):
          Usuario: 
      clave (Cadena de caracteres oculta):
          Clave: 
      servicioFuente (Cadena):
          Servicio Fuente (SubModelo Levantamiento Catastral):
      versionFuente {Cadena}:
          Versión Fuente:
      servicioDestino (Cadena):
          Servicio Destino (Encuesta Survey123):
      usuarioCampo (Cadena):
          Usuario Reconocedor: 
      idsPrecarga (Cadena):
          Números Prediales a cargar:
      informeDetallado {Booleano}:
          Informe Detallado:"""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.HerramientaPrecargarEncuestas_ImportarSurvey(*gp_fixargs((portal, usuario, clave, servicioFuente, versionFuente, servicioDestino, usuarioCampo, idsPrecarga, informeDetallado), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject


import setuptools
 
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()    
 
setuptools.setup(name='ImportarSurvey', 
        version='1.4.0.0',
        author='Esri Colombia',
        author_email='mtorres@esri.co',
        license='MIT',
        long_description=long_description,
        long_description_content_type="text/markdown",
        url='https://dev.azure.com/esrico-con-uaecd-ladm/ImportarEncuestaActualizacionCatastral',
        description=('Módulo de Geoprocesamiento ArcGIS Pro para la Precarga e Inyeccion de Encuestas de Actualización Catastral'),
        python_requires= '>=3.7',
        classifiers=[
            'Programming Language :: Python :: 3.7',
            'License :: OSI Approved :: MIT License',
            'Operating System :: Microsoft :: Windows :: Windows 10',
            'Operating System :: MacOS',
            'Operating System :: Other OS',
            'Natural Language :: Spanish'
        ],
        package_dir={"": "src"},
        packages = setuptools.find_packages(where="src"),
        package_data={'ImportarSurvey':[
                    '*.json',
                    'esri/toolboxes/*',  
                    'esri/arcpy/*', 
                    'esri/help/gp/*',  
                    'esri/help/gp/toolboxes/*', 
                    'esri/help/gp/messages/*'] 
                    }, 
      )


import requests

from arcgis.gis import GIS

class ArcGISRestApiDataAccess :

    def __init__(self, espacioFuente) :
        self.espacioFuente = espacioFuente

    def __search_da(self, tabla_origen, campos, filtro = None ) :
        outFields = str.join(',', campos)
        if filtro == None  :
            queryString = "outFields={}&where=1%3D1".format(outFields)
        else :
            queryString = "outFields={}&where={}".format(outFields,filtro)

        url = "{}/query?f=json&{}".format(tabla_origen, queryString)

        response = requests.get(url)
        json = response.json()
        features_in = json["features"]
        features = []

        for f in features_in :
            feature = f["attributes"]
            features.append(feature)


        return features

    def query(self, table, fields, filter = None) :
        return self.__search_da(table, fields, filter)




# -*- coding: utf-8 -*-

from arcgis.apps import workforce
from ImportarSurvey.Utiles import ToolboxLogger, Configuracion,ToolboxLogger
from ImportarSurvey.ArcGISPythonApiDataAccess import ArcGISPythonApiDataAccess

class ConexionWorkforce():

    def __init__(self, rutaConfiguracion, portal, usuario, clave, workforceProjectID):

        self.rutaConfiguracion = rutaConfiguracion
        self.config = Configuracion(rutaConfiguracion)

        self.portal = portal
        self.usuario = usuario
        self.clave = clave
        self.workforceProjectID = workforceProjectID

    @ToolboxLogger.log_method
    def openWorkforceProject(self, projectID):
        itemProject = self.__gis.content.get(projectID)
        project = workforce.Project(itemProject)
        return project
        
    @ToolboxLogger.log_method
    def Ejecutar(self, servicioDestino) :
        try: 
            ToolboxLogger.info("Portal: {}".format(self.portal))
            ToolboxLogger.info("ID Proyecto Workforce: {}".format(self.workforceProjectID))
            ToolboxLogger.info("Servicio Destino: {}".format(servicioDestino))
            ToolboxLogger.info("Usuario: {}".format(self.usuario))
            ToolboxLogger.info("Clave: {}".format(self.clave))
            
            total_encuestas = 0
            total_registros = 0
            total_errores = 0

            workForce_da = ArcGISPythonApiDataAccess(self.portal, self.usuario, self.clave)
            workforceProject = workForce_da.openWorkforceProject(self.workforceProjectID)

            destino_da = ArcGISPythonApiDataAccess(self.portal, self.usuario, self.clave)
            destino_da.setFeatureService(self.servicioDestino)          
            

            return total_encuestas, total_registros, total_errores, 

        except Exception as e:
            ToolboxLogger.error("Error: {}".format(e))
        finally :
            ToolboxLogger.info("Terminado.")
            
        return total_encuestas, total_registros, total_errores
    
    
    @ToolboxLogger.log_method
    def CrearAsignacionWF(self, dataWFM) :
        assignmentsAdded = None
        try: 
            ToolboxLogger.info("Portal: {}".format(self.portal))
            ToolboxLogger.info("Usuario: {}".format(self.usuario))
            ToolboxLogger.info("Clave: {}".format(self.clave))
            ToolboxLogger.info("ID Proyecto Workforce: {}".format(self.workforceProjectID))

            total_encuestas = 0
            total_registros = 0
            total_errores = 0

            workForce_da = ArcGISPythonApiDataAccess(self.portal, self.usuario, self.clave)
            workforceProject = workForce_da.openWorkforceProject(self.workforceProjectID)
            assignments = []
            assignments.append(
                workforce.Assignment(
                    workforceProject,
                    version="tales",
                    assignment_type="Actualización",
                    work_order_id = dataWFM.rows[0]['id_solicitud'],
                    status="unassigned",
                    location="Terreno: " + dataWFM.rows[0]['cedula_catastral']
                )
            )
            assignmentsAdded = workforceProject.assignments.batch_add(assignments)

        except Exception as e:
            ToolboxLogger.error("Error: {}".format(e))
        finally :
            ToolboxLogger.info("Terminado.")
            
        return assignmentsAdded
                    


# -*- coding: utf-8 -*-

import os

from arcpy import da
from ImportarSurvey.Utiles import ToolboxLogger
from ImportarSurvey.DataAccess import DataAccess

class ArcpyDataAccess(DataAccess) :
    def __init__(self, ruta_espacio) :
        DataAccess.__init__(self)
        self.ruta_espacio = ruta_espacio

    def _getValue(self, cursor, row, fieldName):
        fields = tuple(f.lower() for f in cursor.fields)
        index = fields.index(fieldName.lower())
        if index > -1:
            return row[index]

    def _setValue(self, cursor, row, fieldName, value):
        fields = tuple(f.lower() for f in cursor.fields)
        index = fields.index(fieldName.lower())
        if index > -1:
            row[index] = value

    def _search_da(self, tabla_origen, campos, filtro=None) :
        ToolboxLogger.debug("--> ArcpyDataAccess._search_da")
        tabla_origen = os.path.join(self.ruta_espacio, tabla_origen)
        try:
            if filtro == None:
                cursor_origen = da.SearchCursor(tabla_origen, campos)
            else:
                cursor_origen = da.SearchCursor(tabla_origen, campos, filtro)

            registros_salida = []
            fields = tuple(f for f in cursor_origen.fields)

            for registro_origen in cursor_origen:
                registro_salida = {}
                for f in cursor_origen.fields:
                    index = fields.index(f)
                    registro_salida[f] = registro_origen[index]
                registros_salida.append(registro_salida)

            return registros_salida
        except Exception as e:
            ToolboxLogger.debug("ERROR: ---->{}".format(e))

    def query(self, table, fields, filter = None) :
        return self._search_da(table, fields, filter)
